"""桌面环境配置"""

from typing import NamedTuple


class IntRange(NamedTuple):
    """整数范围"""

    min: int
    """最小值"""

    max: int
    """最大值"""


WINDOWS_CURSOR_SIZE_RANGE = IntRange(1, 48)
"""Windows 鼠标指针大小有效值范围"""

LINUX_CURSOR_SIZE_RANGE = IntRange(16, 96)
"""Windows 鼠标指针大小有效值范围"""


def check_windows_cursor_size_value(
    value: int,
) -> int:
    """检查设置的 Windows 鼠标指针大小是否符合范围

    Args:
        value (int): 鼠标指针大小
    Raises:
        TypeError: 鼠标指针大小不是 int 时
        ValueError: 鼠标指针大小超过有效范围时
    """
    rng = WINDOWS_CURSOR_SIZE_RANGE
    if not isinstance(value, int):
        raise TypeError(f"Windows 鼠标指针大小值应为 int 类型, 但得到 {type(value).__name__} 类型")
    if rng.min <= value <= rng.max:
        return value
    raise ValueError(f"Windows 鼠标指针大小的值 {value} 超过有效范围 [{rng.min}, {rng.max}]")


def check_linux_cursor_size_value(
    value: int,
) -> int:
    """检查设置的 Linux 鼠标指针大小是否符合范围

    Args:
        value (int): 鼠标指针大小
    Raises:
        TypeError: 鼠标指针大小不是 int 时
        ValueError: 鼠标指针大小超过有效范围时
    """
    rng = LINUX_CURSOR_SIZE_RANGE
    if not isinstance(value, int):
        raise TypeError(f"Linux 鼠标指针大小值应为 int 类型, 但得到 {type(value).__name__} 类型")
    if rng.min <= value <= rng.max:
        return value
    raise ValueError(f"Linux 鼠标指针大小的值 {value} 超过有效范围 [{rng.min}, {rng.max}]")



"""配置文件解析"""

import re
from pathlib import Path
from typing import (
    TypedDict,
    TypeAlias,
    cast,
)


from ani2xcur.logger import get_logger
from ani2xcur.config import (
    LOGGER_LEVEL,
    LOGGER_COLOR,
    LOGGER_NAME,
)
from ani2xcur.utils import detect_encoding

logger = get_logger(
    name=LOGGER_NAME,
    level=LOGGER_LEVEL,
    color=LOGGER_COLOR,
)


class INFSection(TypedDict, total=False):
    """INF 文件内容结构"""

    var: dict[str, str | list[str]]
    constant: list[str]


ParsedINF: TypeAlias = dict[str, INFSection]  # pylint: disable=invalid-name
"""parsed INF 的类型"""


def parse_inf_text(
    text: str,
) -> ParsedINF:
    """解析 INF 文本
    Args:
        text (str): INF 文本字符串
    Returns:
        ParsedINF: INF 文件结构字典
    """
    result: ParsedINF = {}
    current: str | None = None

    # 按行处理, 去除注释行（以 ; 或 // 开头 或 全为 / 的分隔行）
    for raw in text.splitlines():
        line = raw.strip()
        if not line:
            continue
        if line.startswith(";") or line.startswith("//") or set(line) <= {"/"}:
            continue

        m = re.match(r"^\[(.+?)\]$", line)
        if m:
            current = m.group(1)
            result[current] = cast(INFSection, {"var": {}, "constant": []})
            continue

        if current is None:
            continue

        if "=" in line:
            # 变量 = 值 (值可能包含逗号和引号)
            key, rhs = line.split("=", 1)
            key = key.strip()
            rhs_raw = rhs.strip()

            # 如果整段值被一对外层引号包裹 -> 去掉外层引号, 作为单一字符串
            if rhs_raw.startswith('"') and rhs_raw.endswith('"'):
                val = rhs_raw[1:-1]
            else:
                # 按逗号分割, 但避免拆分引号内的逗号 (使用正则)
                parts = [p.strip() for p in re.split(r',(?=(?:[^"]*"[^"]*")*[^"]*$)', rhs_raw)]
                # 如果只有一项则返回字符串, 否则返回列表
                # 注意: 不去掉内部项的引号 (除非整体被引号包裹)
                val = parts[0] if len(parts) == 1 else parts

            result[current]["var"][key] = val
        else:
            # 常量行: 单独一项，若被外层引号包裹则去除外层引号
            const = line
            if const.startswith('"') and const.endswith('"'):
                const = const[1:-1]
            result[current]["constant"].append(const)

    return result


def parse_inf_file(
    path: Path,
) -> ParsedINF:
    """解析 INF 文件
    Args:
        path (str): INF 文件路径
    Returns:
        ParsedINF: INF 文件结构字典
    """
    with open(path, "r", encoding=detect_encoding(path), errors="ignore") as f:
        return parse_inf_text(f.read())



"""日志工具"""

import sys
import copy
import inspect
import logging


class LoggingColoredFormatter(logging.Formatter):
    """Logging 格式化类

    Attributes:
        color (bool): 是否启用日志颜色
        COLORS (dict[str, str]): 颜色类型字典
    """

    COLORS = {
        "DEBUG": "\033[0;36m",  # CYAN
        "INFO": "\033[0;32m",  # GREEN
        "WARNING": "\033[0;33m",  # YELLOW
        "ERROR": "\033[0;31m",  # RED
        "CRITICAL": "\033[0;37;41m",  # WHITE ON RED
        "RESET": "\033[0m",  # RESET COLOR
    }

    def __init__(
        self,
        fmt: str | None = None,
        datefmt: str | None = None,
        color: bool | None = True,
    ) -> None:
        """Logging 初始化

        Args:
            fmt (str | None): 日志消息的格式字符串
            datefmt (str | None): 日期 / 时间的显示格式
            color (bool | None): 是否启用彩色日志输出. 默认为 True
        """
        super().__init__(fmt, datefmt)
        self.color = color

    def format(
        self,
        record: logging.LogRecord,
    ) -> str:
        colored_record = copy.copy(record)
        levelname = colored_record.levelname

        if self.color:
            seq = self.COLORS.get(levelname, self.COLORS["RESET"])
            colored_record.levelname = f"{seq}{levelname}{self.COLORS['RESET']}"

        return super().format(colored_record)


def get_logger(
    name: str | None = None,
    level: int | None = logging.INFO,
    color: bool | None = True,
) -> logging.Logger:
    """获取 Loging 对象

    Args:
        name (str | None): Logging 名称
        level (int | None): 日志级别
        color (bool | None): 是否启用彩色日志
    Returns:
        logging.Logger: Logging 对象
    """
    if name is not None:
        log_format = "[%(name)s]-|%(asctime)s|-%(levelname)s: %(message)s"
        logger_name = name
    else:
        frame = inspect.currentframe().f_back
        module = inspect.getmodule(frame)
        logger_name = module.__name__ if module else "root"
        log_format = "[%(name)s:%(funcName)s:%(lineno)d]-|%(asctime)s|-%(levelname)s: %(message)s"

    logger = logging.getLogger(logger_name)
    logger.propagate = False

    if not logger.handlers:
        handler = logging.StreamHandler(sys.stdout)
        handler.setFormatter(
            LoggingColoredFormatter(
                fmt=log_format,
                datefmt=r"%H:%M:%S",
                color=color,
            )
        )
        logger.addHandler(handler)

    logger.setLevel(level)
    fn, lno, func, _ = logger.findCaller(stack_info=False, stacklevel=2)
    logger.debug("Logger 初始化完成, 位置: %s:%s in %s", fn, lno, func)
    return logger



"""Ani2xcur 补丁"""

from ani2xcur.logger import get_logger
from ani2xcur.config import (
    LOGGER_COLOR,
    LOGGER_LEVEL,
    LOGGER_NAME,
)


logger = get_logger(
    name=LOGGER_NAME,
    level=LOGGER_LEVEL,
    color=LOGGER_COLOR,
)


def patch_win2xcur() -> None:
    """将补丁应用到 win2xcur 中"""
    try:
        import win2xcur  # pylint: disable=import-outside-toplevel
    except ImportError as e:
        logger.error("导入 win2xcur 模块发生错误, 无法应用补丁: %s\n这可能因 ImageMagick 未安装导致的问题, 请使用 Ani2xcur 的 ImageMagick 安装功能进行修复", e)
        raise e

    try:
        import win2xcur.scale  # pylint: disable=import-outside-toplevel
    except ImportError:
        from ani2xcur.win2xcur import scale  # pylint: disable=import-outside-toplevel

        win2xcur.scale = scale



"""下载工具"""

import hashlib
from pathlib import Path
from urllib.parse import urlparse

from tqdm import tqdm

from ani2xcur.logger import get_logger
from ani2xcur.config import (
    LOGGER_COLOR,
    LOGGER_LEVEL,
    LOGGER_NAME,
)


logger = get_logger(
    name=LOGGER_NAME,
    level=LOGGER_LEVEL,
    color=LOGGER_COLOR,
)


def download_file_from_url(
    url: str,
    save_path: Path | None = None,
    file_name: str | None = None,
    progress: bool | None = True,
    hash_prefix: str | None = None,
    re_download: bool | None = False,
) -> Path:
    """使用 requrests 库下载文件

    Args:
        url (str): 下载链接
        save_path (Path | None): 下载路径
        file_name (str | None): 保存的文件名, 如果为`None`则从`url`中提取文件
        progress (bool | None): 是否启用下载进度条
        hash_prefix (str | None): sha256 十六进制字符串, 如果提供, 将检查下载文件的哈希值是否与此前缀匹配, 当不匹配时引发`ValueError`
        re_download (bool): 强制重新下载文件
    Returns:
        Path: 下载的文件路径
    Raises:
        ValueError: 当提供了 hash_prefix 但文件哈希值不匹配时
    """
    import requests

    if save_path is None:
        save_path = Path.cwd()

    if not file_name:
        parts = urlparse(url)
        file_name = Path(parts.path).name

    cached_file = save_path.resolve() / file_name

    if re_download or not cached_file.exists():
        save_path.mkdir(parents=True, exist_ok=True)
        temp_file = save_path / f"{file_name}.tmp"
        logger.info("下载 '%s' 到 '%s' 中", file_name, cached_file)
        response = requests.get(url, stream=True, timeout=60)
        response.raise_for_status()
        total_size = int(response.headers.get("content-length", 0))
        with tqdm(
            total=total_size,
            unit="B",
            unit_scale=True,
            desc=file_name,
            disable=not progress,
        ) as progress_bar:
            with open(temp_file, "wb") as file:
                for chunk in response.iter_content(chunk_size=1024):
                    if chunk:
                        file.write(chunk)
                        progress_bar.update(len(chunk))

        if hash_prefix and not compare_sha256(temp_file, hash_prefix):
            logger.error("'%s' 的哈希值不匹配, 正在删除临时文件", temp_file)
            temp_file.unlink()
            raise ValueError(f"文件哈希值与预期的哈希前缀不匹配: {hash_prefix}")

        temp_file.rename(cached_file)
        logger.info("'%s' 下载完成", file_name)
    else:
        logger.info("'%s' 已存在于 '%s' 中", file_name, cached_file)
    return cached_file


def compare_sha256(
    file_path: str | Path,
    hash_prefix: str,
) -> bool:
    """检查文件的 sha256 哈希值是否与给定的前缀匹配

    Args:
        file_path (str | Path): 文件路径
        hash_prefix (str): 哈希前缀
    Returns:
        bool: 匹配结果
    """
    hash_sha256 = hashlib.sha256()
    blksize = 1024 * 1024

    with open(file_path, "rb") as f:
        for chunk in iter(lambda: f.read(blksize), b""):
            hash_sha256.update(chunk)
    return hash_sha256.hexdigest().startswith(hash_prefix.strip().lower())



"""Ani2xcur 更新工具"""

import sys
import importlib.metadata
from pathlib import Path
from datetime import (
    datetime,
    timedelta,
)

from ani2xcur.cmd import run_cmd
from ani2xcur.logger import get_logger
from ani2xcur.config import (
    LOGGER_COLOR,
    LOGGER_LEVEL,
    LOGGER_NAME,
    ANI2XCUR_REPOSITORY_URL,
    WIN2XCUR_REPOSITORY_URL,
    ANI2XCUR_CONFIG_PATH,
)
from ani2xcur.file_operations.file_manager import remove_files


logger = get_logger(
    name=LOGGER_NAME,
    level=LOGGER_LEVEL,
    color=LOGGER_COLOR,
)


def self_update(
    install_from_source: bool | None = False,
    ani2xcur_source: str | None = None,
    win2xcur_source: str | None = None,
    enable_log: bool | None = True,
) -> None:
    """更新 Ani2xcur

    Args:
        install_from_source (bool | None): 是否从源码进行安装
        ani2xcur_source (str | None): Ani2xcur 源仓库的 Git 链接
        win2xcur_source (str | None): win2xcur 源仓库的 Git 链接
        enable_log (bool | None): 是否显示更新日志
    Raises:
        RuntimeError: 更新失败时
    """

    if ani2xcur_source is None:
        ani2xcur_source = ANI2XCUR_REPOSITORY_URL

    if win2xcur_source is None:
        win2xcur_source = WIN2XCUR_REPOSITORY_URL

    cmd = [Path(sys.executable).as_posix(), "-m", "pip", "install", "--upgrade"]
    if install_from_source:
        cmd += [f"git+{ani2xcur_source}", f"git+{win2xcur_source}"]
    else:
        cmd += ["ani2xcur"]

    try:
        logger.info("更新 Ani2xcur 中")
        run_cmd(cmd, live=enable_log)
        logger.info("Ani2xcur 更新成功")
    except RuntimeError as e:
        logger.error("更新 Ani2xcur 时发生错误: %s", e)
        raise RuntimeError(f"更新 Ani2xcur 时发生错误: {e}") from e


def auto_check_update() -> None:
    """检查 Ani2xcur 的更新并提示信息"""
    import requests

    if not check_update_time():
        return

    logger.debug("检查 Ani2xcur 更新中")
    current = importlib.metadata.version("ani2xcur")

    try:
        response = requests.get(url="https://pypi.org/pypi/ani2xcur/json", timeout=2)
        json_data = response.json()
        latest = json_data["info"]["version"]
    except requests.exceptions.ConnectTimeout as e:
        logger.error("获取 Ani2xcur 版本信息时发生错误: %s", e)
        return
    except requests.exceptions.JSONDecodeError as e:
        logger.error("解析 Ani2xcur 版本信息时发生错误: %s", e)
        return

    if current != latest:
        logger.info("新版 Ani2xcur (%s) 可用, 当前使用的版本为 %s\n更新 Ani2xcur 请使用命令: ani2xcur update", latest, current)


def check_update_time() -> bool:
    """检查是否到达检查更新时间

    Args:
        bool: 当时间间隔达到检查更新要求时
    """
    ANI2XCUR_CONFIG_PATH.mkdir(parents=True, exist_ok=True)
    time_record_file = ANI2XCUR_CONFIG_PATH / "update_time"
    current = datetime.now()
    time_span = timedelta(hours=24)
    time_format = r"%Y-%m-%d %H:%M:%S"

    def _save_time_record(
        _record: datetime,
        _time_record_file: Path,
    ) -> None:
        try:
            with open(_time_record_file, "w", encoding="utf-8") as f:
                f.write(_record.strftime(time_format))
        except (PermissionError, OSError) as e:
            logger.debug("保存检查更新时间记录发生错误: %s", e)

    # 清除无效的更新时间记录文件夹
    if time_record_file.is_dir():
        remove_files(time_record_file)

    # 读取上次更新时间记录
    if time_record_file.is_file():
        try:
            with open(time_record_file, "r", encoding="utf-8") as f:
                record = datetime.strptime(f.read(), time_format)
                logger.debug("上次检查更新时间: %s", record)
        except (PermissionError, OSError, ValueError):
            logger.debug("解析更新时间记录失败, 尝试覆盖时间记录文件")
            record = datetime.now()
            _save_time_record(record, time_record_file)
    else:
        logger.debug("未找到更新时间记录文件, 尝试生成中")
        record = datetime.now() - time_span
        _save_time_record(record, time_record_file)

    # 检查上次更新时间与当前时间的间隔
    if current - record >= time_span:
        _save_time_record(current, time_record_file)
        return True

    return False



"""入口文件"""

import typer

from ani2xcur.cli.cli import typer_factory
from ani2xcur.cli.system import (
    version,
    update,
    env,
)
from ani2xcur.cli.convert import (
    win2xcur,
    x2wincur,
)
from ani2xcur.cli.cursor import (
    install_cursor,
    uninstall_cursor,
    export_cursor,
    set_cursor_theme,
    set_cursor_size,
    list_cursor,
    get_current_cursor,
)
from ani2xcur.cli.image_magick import (
    install_image_magick,
    uninstall_image_magick,
)
from ani2xcur.updater import auto_check_update


def get_app() -> typer.Typer:
    """获取 Ani2xcur 命令行应用

    Returns:
        typer.Typer: Ani2xcur 命令行应用
    """
    app = typer_factory("鼠标指针转换、管理、和管理 ImageMagick 的命令行工具")
    app.command(help="显示 Ani2xcur 和其他组件的当前版本", name="version")(version)
    app.command(help="更新 Ani2xcur", name="update")(update)
    app.command(help="列出 Ani2xcur 所使用的环境变量", name="env")(env)

    convert_cli = typer_factory(help="鼠标指针转换工具")
    convert_cli.command(help="将 Windows 鼠标指针文件包转换为 Linux 鼠标指针文件包", name="win2x")(win2xcur)
    convert_cli.command(help="将 Linux 鼠标指针文件包转换为 Windows 鼠标指针文件包", name="x2win")(x2wincur)

    cursor_cli = typer_factory(help="系统鼠标指针管理工具")
    cursor_cli.command(help="将鼠标指针安装到系统中", name="install")(install_cursor)
    cursor_cli.command(help="删除系统中指定的鼠标指针", name="uninstall")(uninstall_cursor)
    cursor_cli.command(help="将鼠标指针从系统中导出", name="export")(export_cursor)
    cursor_cli.command(help="列出当前系统中已安装的鼠标指针", name="list")(list_cursor)
    cursor_cli.command(help="显示当前系统中设置的鼠标指针名称和大小", name="status")(get_current_cursor)

    cursor_set_cli = typer_factory(help="设置鼠标指针的工具")
    cursor_set_cli.command(help="设置系统要使用的鼠标指针主题", name="theme")(set_cursor_theme)
    cursor_set_cli.command(help="设置系统要使用的鼠标指针大小", name="size")(set_cursor_size)
    cursor_cli.add_typer(cursor_set_cli, name="set")

    image_magick_cli = typer_factory(help="ImageMagick 管理工具")
    image_magick_cli.command(help="安装 ImageMagick 到系统中", name="install")(install_image_magick)
    image_magick_cli.command(help="将 ImageMagick 从系统中卸载", name="uninstall")(uninstall_image_magick)

    app.add_typer(convert_cli, name="convert")
    app.add_typer(cursor_cli, name="cursor")
    app.add_typer(image_magick_cli, name="imagemagick")

    return app


def main() -> None:
    """主函数"""
    get_app()()
    auto_check_update()


if __name__ == "__main__":
    main()



"""Windows 桌面环境配置工具"""

import ctypes
import os
import re
from pathlib import Path
from typing import Literal

try:
    import win32gui
    import win32con
    import win32com.client
except ImportError:
    win32gui = NotImplemented
    win32con = NotImplemented
    win32com = NotImplemented  # pylint: disable=invalid-name

from ani2xcur.manager.regedit import (
    registry_query_value,
    registry_set_value,
    registry_enum_values,
    RegistryAccess,
    RegistryRootKey,
    RegistryValueType,
)
from ani2xcur.utils import (
    extend_list_to_length,
    lowercase_dict_keys,
)
from ani2xcur.manager.base import CURSOR_KEYS

WINDOWS_CURSOR_CURSORS_PATH = r"Control Panel\Cursors"
"""Windows 鼠标指针配置路径"""

WINDOWS_CURSOR_CURSORS_SCHEME_PATH = r"Control Panel\Cursors\Schemes"
"""Windows 鼠标指针方案路径"""

WINDOWS_ACCESSIBILITY_PATH = r"Software\Microsoft\Accessibility"
"""Windows 无障碍配置路径"""


def has_var_string(
    text: str,
) -> bool:
    """检测字符串中是否包含 %var% 格式的变量

    Args:
        text (str): 需要检测的字符串
    Returns:
        bool: 如果字符串中存在 %var% 格式的变量则返回 True, 否则返回 False
    """
    pattern = r"%([^%]+)%"
    return bool(re.search(pattern, text))


def expand_var_string(
    text: str,
    vars_dict: dict[str, str] = None,
) -> str:
    """将字符串中的 %var% 变量进行替换, 并优先查找变量表中的值

    Args:
        text (str): 需要处理的字符串
        vars_dict (dict[str, str] | None): 变量字典
    Returns:
        str: 处理后的字符串
    """

    def _replace_env_var(
        match: re.Match,
    ) -> str:
        env_var = match.group(1)
        env_var_lower = match.group(1).lower().strip()
        return vars_dict.get(env_var_lower, os.environ.get(env_var, match.group(0)))

    if vars_dict is None:
        vars_dict = {}

    pattern = r"%([^%]+)%"
    text = text.replace(r"%10%", r"%SYSTEMROOT%")
    vars_dict = lowercase_dict_keys(vars_dict)
    result = re.sub(pattern, _replace_env_var, text)
    return result


def refresh_system_params() -> None:
    """通知系统刷新设置以应用更改"""
    ctypes.windll.user32.SystemParametersInfoW(0x0057, 0, None, 0x01 | 0x02)


def get_windows_cursor_theme() -> str:
    """获取 Windows 桌面当前使用的鼠标指针配置名称

    Returns:
        (str | None): 当前使用的鼠标指针名称
    """
    return registry_query_value(
        name="",
        sub_key=WINDOWS_CURSOR_CURSORS_PATH,
        key=RegistryRootKey.CURRENT_USER,
        access=RegistryAccess.READ,
    )


def get_windows_cursor_size() -> int | None:
    """获取 Windows 桌面当前使用的鼠标指针大小

    Returns:
        (int | None): 当前使用的鼠标指针大小
    """
    return registry_query_value(
        name="CursorSize",
        sub_key=WINDOWS_ACCESSIBILITY_PATH,
        key=RegistryRootKey.CURRENT_USER,
        access=RegistryAccess.READ,
    )


def set_windows_cursor_theme(
    cursor_name: str,
) -> None:
    """设置 Windows 桌面当前使用的鼠标指针配置名称

    Args:
        cursor_name (str): 要设置的鼠标指针配置名称
    """
    # 获取鼠标指针方案列表
    schemes = registry_enum_values(
        sub_key=WINDOWS_CURSOR_CURSORS_SCHEME_PATH,
        key=RegistryRootKey.CURRENT_USER,
        access=RegistryAccess.READ,
    )
    if cursor_name not in schemes:
        return

    scheme_data = schemes[cursor_name]
    cursor_paths = [x for x in extend_list_to_length(scheme_data.split(","), target_length=len(CURSOR_KEYS["win"]))]

    # 设置方案名称
    registry_set_value(
        name="",
        value=cursor_name,
        reg_type=RegistryValueType.SZ,
        sub_key=WINDOWS_CURSOR_CURSORS_PATH,
        key=RegistryRootKey.CURRENT_USER,
        access=RegistryAccess.SET_VALUE,
    )

    # 设置鼠标指针对应的文件路径
    for cursor, path in zip(CURSOR_KEYS["win"], cursor_paths):
        reg_type = RegistryValueType.EXPAND_SZ if has_var_string(path) else RegistryValueType.SZ
        registry_set_value(
            name=cursor,
            value=path,
            reg_type=reg_type,
            sub_key=WINDOWS_CURSOR_CURSORS_PATH,
            key=RegistryRootKey.CURRENT_USER,
            access=RegistryAccess.SET_VALUE,
        )

    refresh_system_params()


def set_windows_cursor_size(
    cursor_size: int,
) -> None:
    """设置 Windows 桌面当前使用的鼠标指针大小

    Args:
        cursor_size (int): 要设置的鼠标指针大小
    """
    registry_set_value(
        name="CursorSize",
        value=cursor_size,
        reg_type=RegistryValueType.DWORD,
        sub_key=WINDOWS_ACCESSIBILITY_PATH,
        key=RegistryRootKey.CURRENT_USER,
        access=RegistryAccess.SET_VALUE,
    )
    refresh_system_params()


def broadcast_settings_change(
    area_name: Literal["Environment", "Policy", "intl"] | None = "Environment",
) -> bool:
    """发送 WM_SETTINGCHANGE 广播消息通知系统设置已更改

    Args:
        area_name (Literal["Environment", "Policy", "intl"] | None):
            更改的区域名称
            - 如果是环境变量, 传入 `Environment`
            - 如果是通过注册表更改了策略或通用设置, 传入 `Policy`
            - 如果是鼠标/字体等, 传入 `intl` 或留空
    Returns:
        bool: 通知结果
    """
    # WM_SETTINGCHANGE 的消息数值在 win32con 中已定义
    # HWND_BROADCAST: 0xFFFF (发送给所有顶层窗口)
    # SMTO_ABORTIFHUNG: 如果目标窗口挂起，则立即返回

    result = win32gui.SendMessageTimeout(  # pylint: disable=c-extension-no-member,no-member
        win32con.HWND_BROADCAST,
        win32con.WM_SETTINGCHANGE,
        0,
        area_name,
        win32con.SMTO_ABORTIFHUNG,
        5000,  # 等待每个窗口响应的最大毫秒数
    )
    return result


def create_windows_shortcut(
    target_path: Path,
    shortcut_path: Path,
    description: str | None = "",
    working_dir: Path | None = None,
    icon_path: Path | None = None,
) -> None:
    """创建 Windows 快捷方式

    Args:
        target_path (Path): 目标文件路径
        shortcut_path (Path): 快捷方式保存路径 (应以 .lnk 结尾)
        description (str | None): 快捷方式描述
        working_dir (Path | None): 工作目录
        icon_path (Path | None): 图标路径
    """
    shell = win32com.client.Dispatch("WScript.Shell")
    shortcut = shell.CreateShortCut(str(shortcut_path))
    shortcut.Targetpath = str(target_path)
    shortcut.WorkingDirectory = str(working_dir) or str(target_path.parent)
    if description:
        shortcut.Description = description
    if icon_path is not None:
        shortcut.IconLocation = str(icon_path)
    shortcut.save()



"""ImageMagick 管理工具"""

import sys
import traceback
from typing import Annotated
from pathlib import Path

import typer

from ani2xcur.config import (
    LOGGER_NAME,
    LOGGER_LEVEL,
    LOGGER_COLOR,
    IMAGE_MAGICK_WINDOWS_INSTALL_PATH,
)
from ani2xcur.logger import get_logger
from ani2xcur.manager.image_magick_manager import (
    install_image_magick_windows,
    install_image_magick_linux,
    uninstall_image_magick_windows,
    uninstall_image_magick_linux,
)
from ani2xcur.utils import (
    is_admin_on_windows,
    is_root_on_linux,
)

logger = get_logger(
    name=LOGGER_NAME,
    level=LOGGER_LEVEL,
    color=LOGGER_COLOR,
)


def install_image_magick(
    install_path: Annotated[
        Path | None,
        typer.Option(
            help="(仅 Windows 平台) 自定义安装 ImageMagick 的目录",
            resolve_path=True,
        ),
    ] = None,
    force: Annotated[
        bool,
        typer.Option(
            "--yes",
            "-y",
            help="直接确认安装",
        ),
    ] = False,
) -> None:
    """安装 ImageMagick 到系统中"""
    if sys.platform == "win32":
        if not is_admin_on_windows():
            logger.error("当前未使用管理员权限运行 Ani2xcur, 无法安装 ImageMagick, 请使用管理员权限启动 Ani2xcur")
            sys.exit(1)

        if install_path is None:
            install_path = IMAGE_MAGICK_WINDOWS_INSTALL_PATH
            logger.info("未使用 --install-path 参数指定 ImageMagick 安装路径, 使用默认安装路径: '%s'", install_path)
        else:
            logger.info("安装 ImageMagick 的路径: '%s'", install_path)

        if not force:
            typer.confirm("确认安装 ImageMagick 吗?", abort=True)
        try:
            install_image_magick_windows(install_path=install_path)
        except PermissionError as e:
            traceback.print_exc()
            logger.error("安装 ImageMagick 时发生错误: %s\n请检查是否使用管理员权限运行 Ani2xcur", e)
            sys.exit(1)
    elif sys.platform == "linux":
        if not is_root_on_linux():
            logger.error("当前未使用 root 权限运行 Ani2xcur, 无法安装 ImageMagick, 请使用 root 权限启动 Ani2xcur")
            sys.exit(1)

        if not force:
            typer.confirm("确认安装 ImageMagick 吗?", abort=True)

        try:
            install_image_magick_linux()
        except RuntimeError as e:
            traceback.print_exc()
            logger.error("安装 ImageMagick 时发生错误: %s\n当前的 Linux 发行版可能不支持自动安装 ImageMagick, 请尝试手动安装 ImageMagick", e)
            sys.exit(1)

    else:
        logger.error("不支持的系统: %s", sys.platform)
        sys.exit(1)


def uninstall_image_magick(
    force: Annotated[
        bool,
        typer.Option(
            "--yes",
            "-y",
            help="直接确认卸载",
        ),
    ] = False,
) -> None:
    """将 ImageMagick 从系统中卸载"""
    if sys.platform == "win32":
        if not is_admin_on_windows():
            logger.error("当前未使用管理员权限运行 Ani2xcur, 无法卸载 ImageMagick, 请使用管理员权限启动 Ani2xcur")
            sys.exit(1)

        if not force:
            typer.confirm("确认卸载 ImageMagick 吗?", abort=True)

        try:
            uninstall_image_magick_windows()
        except PermissionError as e:
            traceback.print_exc()
            logger.error("安装 ImageMagick 时发生错误: %s\n请检查是否使用管理员权限运行 Ani2xcur", e)
            sys.exit(1)
    elif sys.platform == "linux":
        if not is_root_on_linux():
            logger.error("当前未使用 root 权限运行 Ani2xcur, 无法卸载 ImageMagick, 请使用 root 权限启动 Ani2xcur")
            sys.exit(1)

        if not force:
            typer.confirm("确认卸载 ImageMagick 吗?", abort=True)

        try:
            uninstall_image_magick_linux()
        except RuntimeError as e:
            traceback.print_exc()
            logger.error("安装 ImageMagick 时发生错误: %s\n当前的 Linux 发行版可能不支持自动安装 ImageMagick, 请尝试手动安装 ImageMagick", e)
            sys.exit(1)
    else:
        logger.error("不支持的系统: %s", sys.platform)
        sys.exit(1)



"""Win2xcur 调用工具"""

from pathlib import Path
from typing import TypedDict

from ani2xcur.logger import get_logger
from ani2xcur.config import (
    LOGGER_COLOR,
    LOGGER_LEVEL,
    LOGGER_NAME,
)
from ani2xcur.utils import (
    open_file_as_bytes,
    save_bytes_to_file,
)
from ani2xcur.win2xcur.patch import patch_win2xcur

logger = get_logger(
    name=LOGGER_NAME,
    level=LOGGER_LEVEL,
    color=LOGGER_COLOR,
)


class Win2xcurArgs(TypedDict, total=False):
    """调用 win2xcur_process() 时输入的参数字典"""

    input_file: Path
    """光标文件的路径"""

    output_path: Path
    """保存转换后的光标文件的路径"""

    save_name: str | None
    """保存的文件名"""

    shadow: bool | None
    """是否模拟 Windows 的阴影效果"""

    shadow_opacity: int | None
    """阴影的不透明度 (0 到 255)"""

    shadow_radius: float | None
    """阴影模糊效果的半径 (宽度的分数值)"""

    shadow_sigma: float | None
    """阴影模糊效果的西格玛值 (宽度的分数值)"""

    shadow_x: float | None
    """阴影的 x 偏移量 (宽度的分数值)"""

    shadow_y: float | None
    """阴影的 y 偏移量 (高度的分数值)"""

    shadow_color: str | None
    """阴影的颜色 (十六进制颜色格式)"""

    scale: float | None
    """按指定倍数缩放光标"""


def win2xcur_process(
    input_file: Path,
    output_path: Path,
    save_name: str | None = None,
    shadow: bool | None = False,
    shadow_opacity: int | None = 50,  # pylint: disable=unused-argument
    shadow_radius: float | None = 0.1,
    shadow_sigma: float | None = 0.1,
    shadow_x: float | None = 0.05,
    shadow_y: float | None = 0.05,
    shadow_color: str | None = "#000000",
    scale: float | None = None,
) -> Path:
    """win2xcur 处理过程

    Args:
        input_file (Path): 光标文件的路径
        output_path (Path): 保存转换后的光标文件的路径
        save_name (str | None): 保存的文件名
        shadow (bool | None): 是否模拟 Windows 的阴影效果
        shadow_opacity (int | None): 阴影的不透明度 (0 到 255)
        shadow_radius (float | None): 阴影模糊效果的半径 (宽度的分数值)
        shadow_sigma (float | None): 阴影模糊效果的西格玛值 (宽度的分数值)
        shadow_x (float | None): 阴影的 x 偏移量 (宽度的分数值)
        shadow_y (float | None): 阴影的 y 偏移量 (高度的分数值)
        shadow_color (str | None): 阴影的颜色 (十六进制颜色格式)
        scale (float | None): 按指定倍数缩放光标
    Returns:
        Path: 光标保存路径
    Raises:
        ValueError: 读取不支持的光标文件格式时
        Exception: 发生未知错误时
    """
    try:
        # 依赖关系: win2xcur -> wand -> ImageMagick
        # 当 ImageMagick 未安装时将导致 win2xcur 导入失败
        patch_win2xcur()
        from win2xcur.scale import apply_to_frames as apply_to_frames_for_scale  # pylint: disable=import-outside-toplevel
        from win2xcur.shadow import apply_to_frames as apply_to_frames_for_shadow  # pylint: disable=import-outside-toplevel
        from win2xcur.parser import open_blob  # pylint: disable=import-outside-toplevel
        from win2xcur.writer import to_x11  # pylint: disable=import-outside-toplevel
    except ImportError as e:
        raise ImportError(f"导入 win2xcur 模块时发生错误: {e}\n这可能因 ImageMagick 未安装导致的问题, 请使用 Ani2xcur 的 ImageMagick 安装功能进行修复") from e

    if save_name is None:
        save_name = input_file.stem

    blob = open_file_as_bytes(input_file)

    try:
        cursor = open_blob(blob)
    except ValueError as e:
        logger.error("不支持的光标文件格式: '%s'", input_file.suffix)
        raise e
    except Exception as e:
        logger.error("打开光标文件时发生未知错误: %s", e)
        raise e

    if scale:
        apply_to_frames_for_scale(cursor.frames, scale=scale)

    if shadow:
        apply_to_frames_for_shadow(
            cursor.frames,
            color=shadow_color,
            radius=shadow_radius,
            sigma=shadow_sigma,
            xoffset=shadow_x,
            yoffset=shadow_y,
        )
    result = to_x11(cursor.frames)
    output_path = output_path / save_name
    save_bytes_to_file(result, output_path)
    return output_path


class X2wincurArgs(TypedDict, total=False):
    """调用 x2wincur_process() 时输入的参数字典"""

    input_file: Path
    """光标文件的路径"""

    output_path: Path
    """保存转换后的光标文件的路径"""

    save_name: str | None
    """保存的文件名 (不包括扩展名)"""

    scale: float | None
    """按指定倍数缩放光标"""


def x2wincur_process(
    input_file: Path,
    output_path: Path,
    save_name: str | None = None,
    scale: float | None = None,
) -> None:
    """x2wincur 处理过程

    Args:
        input_file (Path): 光标文件的路径
        output_path (Path): 保存转换后的光标文件的路径
        save_name (str | None): 保存的文件名 (不包括扩展名)
        scale (float | None): 按指定倍数缩放光标
    Returns:
        Path: 光标保存路径
    Raises:
        ValueError: 读取不支持的光标文件格式时
        Exception: 发生未知错误时
        ImportError: 当导入 win2xcur 模块发生失败时
    """
    try:
        # 依赖关系: win2xcur -> wand -> ImageMagick
        # 当 ImageMagick 未安装时将导致 win2xcur 导入失败
        patch_win2xcur()
        from win2xcur.scale import apply_to_frames as apply_to_frames_for_scale  # pylint: disable=import-outside-toplevel
        from win2xcur.parser import open_blob  # pylint: disable=import-outside-toplevel
        from win2xcur.writer import to_smart  # pylint: disable=import-outside-toplevel
    except ImportError as e:
        raise ImportError(f"导入 win2xcur 模块时发生错误: {e}\n这可能因 ImageMagick 未安装导致的问题, 请使用 Ani2xcur 的 ImageMagick 安装功能进行修复") from e

    if save_name is None:
        save_name = input_file.stem

    blob = open_file_as_bytes(input_file)

    try:
        cursor = open_blob(blob)
    except ValueError as e:
        logger.error("不支持的光标文件格式: '%s'", input_file.suffix)
        raise e
    except Exception as e:
        logger.error("打开光标文件时发生未知错误: %s", e)
        raise e

    if scale:
        apply_to_frames_for_scale(cursor.frames, scale=scale)

    ext, result = to_smart(cursor.frames)
    output_path = output_path / f"{save_name}{ext}"
    save_bytes_to_file(result, output_path)
    return output_path



"""Windows 环境变量管理工具"""

from typing import Literal

from ani2xcur.manager.desktop_config.windows import broadcast_settings_change
from ani2xcur.manager.regedit import (
    RegistryAccess,
    RegistryRootKey,
    RegistryValueType,
    registry_query_value,
    registry_set_value,
    registry_delete_value,
)

WINDOWS_ENV_VALUE_REGESTRY_PATH_SYSTEM = [RegistryRootKey.LOCAL_MACHINE, r"SYSTEM\ControlSet001\Control\Session Manager\Environment"]
"""Windows 的环境变量的注册表路径 (系统变量)"""

WINDOWS_ENV_VALUE_REGESTRY_PATH_USER = [RegistryRootKey.CURRENT_USER, "Environment"]
"""Windows 的环境变量的注册表路径 (用户变量)"""


def add_path_to_env_path(
    new_path: str,
    dtype: Literal["user", "system"] | None = "user",
) -> bool:
    """将路径添加到 PATH 环境变量

    Args:
        new_path (str): 要添加到 PATH 环境变量的路径
        dtype (Literal["user", "system"] | None): 要添加路径的环境变量类型
    Returns:
        bool: 当添加成功时返回 True, 已经存在时则返回 False
    Raises:
        ValueError: 使用未知的环境变量类型时
    """
    if dtype == "user":
        key, sub_key = WINDOWS_ENV_VALUE_REGESTRY_PATH_USER
    elif dtype == "system":
        key, sub_key = WINDOWS_ENV_VALUE_REGESTRY_PATH_SYSTEM
    else:
        raise ValueError(f"未知的环境变量类型: {dtype}")

    raw_path = registry_query_value(
        name="Path",
        sub_key=sub_key,
        key=key,
        access=RegistryAccess.READ,
    )
    paths = [p for p in raw_path.split(";") if p.strip()]
    if new_path in paths:
        return False

    paths.append(new_path)
    registry_set_value(
        name="Path",
        value=";".join(paths),
        reg_type=RegistryValueType.EXPAND_SZ,
        sub_key=sub_key,
        key=key,
        access=RegistryAccess.WRITE,
    )
    broadcast_settings_change()
    return True


def add_val_to_env(
    name: str,
    value: str,
    dtype: Literal["user", "system"] | None = "user",
) -> None:
    """将变量添加到环境变量

    Args:
        name (str): 环境变量的名称
        value (str): 环境变量的值
        dtype (Literal["user", "system"] | None): 要添加路径的环境变量类型
    Raises:
        ValueError: 使用未知的环境变量类型时
    """
    if dtype == "user":
        key, sub_key = WINDOWS_ENV_VALUE_REGESTRY_PATH_USER
    elif dtype == "system":
        key, sub_key = WINDOWS_ENV_VALUE_REGESTRY_PATH_SYSTEM
    else:
        raise ValueError(f"未知的环境变量类型: {dtype}")

    registry_set_value(
        name=name,
        value=value,
        reg_type=RegistryValueType.EXPAND_SZ,
        sub_key=sub_key,
        key=key,
        access=RegistryAccess.WRITE,
    )
    broadcast_settings_change()


def delete_path_from_env_path(
    key_path: str,
    dtype: Literal["user", "system"] | None = "user",
) -> bool:
    """将指定路径从 PATH 环境变量中删除

    Args:
        key_path (str): 要从 PATH 环境变量删除的路径
        dtype (Literal["user", "system"] | None): 要删除路径的环境变量类型
    Raises:
        ValueError: 使用未知的环境变量类型时
    """
    if dtype == "user":
        key, sub_key = WINDOWS_ENV_VALUE_REGESTRY_PATH_USER
    elif dtype == "system":
        key, sub_key = WINDOWS_ENV_VALUE_REGESTRY_PATH_SYSTEM
    else:
        raise ValueError(f"未知的环境变量类型: {dtype}")

    raw_path = registry_query_value(
        name="Path",
        sub_key=sub_key,
        key=key,
        access=RegistryAccess.READ,
    )
    paths = [p for p in raw_path.split(";") if p.strip() and key_path not in p]
    registry_set_value(
        name="Path",
        value=";".join(paths),
        reg_type=RegistryValueType.EXPAND_SZ,
        sub_key=sub_key,
        key=key,
        access=RegistryAccess.WRITE,
    )
    broadcast_settings_change()


def delete_val_from_env(
    name: str,
    dtype: Literal["user", "system"] | None = "user",
) -> None:
    """将变量从环境变量中删除

    Args:
        name (str): 环境变量的名称
        dtype (Literal["user", "system"] | None): 要删除变量的环境变量类型
    Raises:
        ValueError: 使用未知的环境变量类型时
    """
    if dtype == "user":
        key, sub_key = WINDOWS_ENV_VALUE_REGESTRY_PATH_USER
    elif dtype == "system":
        key, sub_key = WINDOWS_ENV_VALUE_REGESTRY_PATH_SYSTEM
    else:
        raise ValueError(f"未知的环境变量类型: {dtype}")

    registry_delete_value(name=name, sub_key=sub_key, key=key, access=RegistryAccess.WRITE)
    broadcast_settings_change()



"""文件操作工具"""

import os
import stat
import shutil
from pathlib import Path

from tqdm import tqdm

from ani2xcur.logger import get_logger
from ani2xcur.config import (
    LOGGER_LEVEL,
    LOGGER_COLOR,
    LOGGER_NAME,
)

logger = get_logger(
    name=LOGGER_NAME,
    level=LOGGER_LEVEL,
    color=LOGGER_COLOR,
)


def remove_files(
    path: Path,
) -> None:
    """文件删除工具，支持删除只读文件和非空文件夹。

    Args:
        path (str | Path): 要删除的文件或目录路径
    Raises:
        ValueError: 路径不存在时
        OSError: 删除过程中的系统错误
    """

    if not path.exists():
        logger.error("路径不存在: '%s'", path)
        raise ValueError(f"要删除的 {path} 路径不存在")

    def _handle_remove_readonly(
        func,
        path_str,
        _,
    ):
        """处理只读文件的错误处理函数"""
        if os.path.exists(path_str):
            os.chmod(path_str, stat.S_IWRITE)
            func(path_str)

    try:
        if path.is_file() or path.is_symlink():
            # 处理文件或符号链接
            os.chmod(path, stat.S_IWRITE)
            path.unlink()

        elif path.is_dir():
            # 处理文件夹
            shutil.rmtree(path, onerror=_handle_remove_readonly)

    except OSError as e:
        logger.error("删除失败: '%s' - 原因: %s", path, e)
        raise e


def copy_files(
    src: Path | str,
    dst: Path | str,
) -> None:
    """复制文件或目录

    Args:
        src (Path | str): 源文件路径
        dst (Path | str): 复制文件到指定的路径
    Raises:
        PermissionError: 没有权限复制文件时
        OSError: 复制文件失败时
        FileNotFoundError: 源文件未找到时
        ValueError: 路径逻辑错误（如循环复制）时
    """
    try:
        # 转换为绝对路径以进行准确的路径比对
        src_path = src.resolve()
        dst_path = dst.resolve()

        # 检查源是否存在
        if not src_path.exists():
            logger.error("源路径不存在: '%s'", src)
            raise FileNotFoundError(f"源路径不存在: {src}")

        # 防止递归复制（例如将目录复制到其自身的子目录中）
        if src_path.is_dir() and dst_path.is_relative_to(src_path):
            logger.error("不能将目录复制到自身或其子目录中: '%s'", src)
            raise ValueError(f"不能将目录复制到自身或其子目录中: {src}")

        # 如果目标是已存在的目录, 则在其下创建同名项
        if dst_path.exists() and dst_path.is_dir():
            dst_file = dst_path / src_path.name
        else:
            dst_file = dst_path

        # 确保目标父目录存在
        dst_file.parent.mkdir(parents=True, exist_ok=True)

        # 复制操作
        if src_path.is_file():
            # copy2 会尽量保留文件元数据
            shutil.copy2(src_path, dst_file)
        else:
            # symlinks=True: 保留软链接本身而非复制指向的内容
            # dirs_exist_ok=True: 实现合并逻辑，如果目标目录已存在则覆盖同名文件
            try:
                shutil.copytree(src_path, dst_file, symlinks=True, dirs_exist_ok=True)
            except shutil.Error:
                # Linux 中遇到已存在的软链接会导致失败, 则使用 symlinks=False 重试
                shutil.copytree(src_path, dst_file, symlinks=False, dirs_exist_ok=True)

    except PermissionError as e:
        logger.error("权限错误, 请检查文件权限或以管理员身份运行: %s", e)
        raise e
    except OSError as e:
        logger.error("复制失败: %s", e)
        raise e
    except Exception as e:
        logger.error("发生非预期错误: %s", e)
        raise e


def get_file_list(
    path: Path,
    resolve: bool | None = False,
    max_depth: int | None = -1,
    show_progress: bool | None = True,
    include_dirs: bool | None = False,
) -> list[Path]:
    """获取当前路径下的所有文件（和可选的目录）的绝对路径

    Args:
        path (Path): 要获取列表的目录
        resolve (bool | None): 将路径进行完全解析, 包括链接路径
        max_depth (int | None): 最大遍历深度, -1 表示不限制深度, 0 表示只遍历当前目录
        show_progress (bool | None): 是否显示 tqdm 进度条
        include_dirs (bool | None): 是否在结果中包含目录路径
    Returns:
        (list[Path]): 路径列表的绝对路径
    """

    if not path or not path.exists():
        return []

    if path.is_file():
        return [path.resolve() if resolve else path.absolute()]

    base_depth = len(path.resolve().parts)

    file_list: list[Path] = []
    with tqdm(desc=f"扫描目录 {path}", position=0, leave=True, disable=not show_progress) as dir_pbar:
        with tqdm(desc="发现条目数", position=1, leave=True, disable=not show_progress) as file_pbar:
            for root, dirs, files in os.walk(path):
                root_path = Path(root)
                current_depth = len(root_path.resolve().parts) - base_depth

                # 超过最大深度则阻止继续向下遍历
                if max_depth != -1 and current_depth >= max_depth:
                    # 如果需要包含目录, 虽然停止深挖, 但当前层的目录仍可加入
                    if include_dirs:
                        for d in dirs:
                            dir_path = root_path / d
                            file_list.append(dir_path.resolve() if resolve else dir_path.absolute())
                            file_pbar.update(1)
                    dirs.clear()
                else:
                    # 如果启用，将当前层级的目录加入列表
                    if include_dirs:
                        for d in dirs:
                            dir_path = root_path / d
                            file_list.append(dir_path.resolve() if resolve else dir_path.absolute())
                            file_pbar.update(1)

                for file in files:
                    file_path = root_path / file
                    file_list.append(file_path.resolve() if resolve else file_path.absolute())
                    file_pbar.update(1)

                dir_pbar.update(1)

    return file_list


def save_create_symlink(
    target: Path,
    link: Path,
) -> None:
    """创建软链接, 当创建软链接失败时则尝试复制文件

    Args:
        target (Path): 源文件路径
        link (Path): 软链接到的目的路径
    """
    try:
        link.symlink_to(target)
        logger.debug("创建软链接: '%s' -> '%s'", target, link)
    except OSError:
        logger.debug("尝试创建软链接失败, 尝试复制文件: '%s' -> '%s'", target, link)
        copy_files(target, link)



"""Windows 注册表操作工具"""

import sys
from enum import (
    IntFlag,
    Enum,
)


if sys.platform == "win32":
    import winreg

    class RegistryRootKey(IntFlag):
        """注册表根路径"""

        CLASSES_ROOT = winreg.HKEY_CLASSES_ROOT
        """HKEY_CLASSES_ROOT: 包含所有已注册的文件类型、扩展名与 COM 类信息"""

        CURRENT_USER = winreg.HKEY_CURRENT_USER
        """HKEY_CURRENT_USER: 当前登录用户的配置分支"""

        LOCAL_MACHINE = winreg.HKEY_LOCAL_MACHINE
        """HKEY_LOCAL_MACHINE: 计算机级别的全局配置"""

        USERS = winreg.HKEY_USERS
        """HKEY_USERS: 所有用户配置的根节点"""

        CURRENT_CONFIG = winreg.HKEY_CURRENT_CONFIG
        """HKEY_CURRENT_CONFIG: 当前硬件配置视图"""

        def __int__(
            self,
        ) -> int:
            return self.value

    class RegistryAccess(IntFlag):
        """注册表访问权限"""

        READ = winreg.KEY_READ
        """读取键值"""

        WRITE = winreg.KEY_WRITE
        """写入 / 修改键值"""

        SET_VALUE = winreg.KEY_SET_VALUE
        """只允许写键值"""

        QUERY_VALUE = winreg.KEY_QUERY_VALUE
        """只允许读键值"""

        CREATE_SUB_KEY = winreg.KEY_CREATE_SUB_KEY
        """允许创建子键"""

        ENUMERATE_SUB_KEYS = winreg.KEY_ENUMERATE_SUB_KEYS
        """允许枚举子键"""

        NOTIFY = winreg.KEY_NOTIFY
        """注册表变更通知"""

        ALL_ACCESS = winreg.KEY_ALL_ACCESS
        """完全控制"""

        WOW64_64KEY = winreg.KEY_WOW64_64KEY
        """强制访问 64 位注册表视图"""

        WOW64_32KEY = winreg.KEY_WOW64_32KEY
        """强制访问 32 位注册表视图"""

        def __int__(
            self,
        ) -> int:
            return self.value

    class RegistryValueType(IntFlag):
        """注册表值类型"""

        SZ = winreg.REG_SZ
        """普通字符串 -> `str`"""

        EXPAND_SZ = winreg.REG_EXPAND_SZ
        """可展开字符串 (含 %PATH% 等变量) -> `str`"""

        DWORD = winreg.REG_DWORD
        """32 位整数 -> `int`"""

        QWORD = winreg.REG_QWORD
        """64 位整数 -> `int`"""

        BINARY = winreg.REG_BINARY
        """二进制数据 -> `bytes`"""

        MULTI_SZ = winreg.REG_MULTI_SZ
        """多字符串数组 -> `list[str]`"""

        NONE = winreg.REG_NONE
        """无类型数据 -> `bytes` | `None`"""

        def __int__(
            self,
        ) -> int:
            return int(self.value)
else:

    class RegistryRootKey(Enum):  # pylint: disable=missing-class-docstring
        CLASSES_ROOT = NotImplemented
        CURRENT_USER = NotImplemented
        LOCAL_MACHINE = NotImplemented
        USERS = NotImplemented
        CURRENT_CONFIG = NotImplemented

    class RegistryAccess(Enum):  # pylint: disable=missing-class-docstring
        READ = NotImplemented
        WRITE = NotImplemented
        SET_VALUE = NotImplemented
        QUERY_VALUE = NotImplemented
        CREATE_SUB_KEY = NotImplemented
        ENUMERATE_SUB_KEYS = NotImplemented
        NOTIFY = NotImplemented
        ALL_ACCESS = NotImplemented
        WOW64_64KEY = NotImplemented
        WOW64_32KEY = NotImplemented

    class RegistryValueType(Enum):  # pylint: disable=missing-class-docstring
        SZ = NotImplemented
        EXPAND_SZ = NotImplemented
        DWORD = NotImplemented
        QWORD = NotImplemented
        BINARY = NotImplemented
        MULTI_SZ = NotImplemented
        NONE = NotImplemented

    winreg = NotImplemented  # pylint: disable=invalid-name


def registry_query_value(
    name: str,
    sub_key: str,
    key: RegistryRootKey | None = RegistryRootKey.CURRENT_USER,
    access: RegistryAccess | None = RegistryAccess.READ,
) -> str | int | bytes | list[str] | None:
    """查询注册表中的键

    Args:
        name (str): 要查询的注册表的值名称
        sub_key (str): 目标注册表子键路径 (不包含根键部分)
        key (RegistryRootKey | None): 注册表根键枚举
        access (RegistryAccess | None): 打开注册表键时使用的访问权限标志
    Returns:
        (str | int | bytes | list[str] | None):
            查询到的注册表值数据，根据注册表类型返回不同 Python 类型：

            - `REG_SZ` / `REG_EXPAND_SZ` -> `str`
            - `REG_DWORD` / `REG_QWORD` -> `int`
            - `REG_BINARY` -> `bytes`
            - `REG_MULTI_SZ` -> `list[str]`
            - 当值不存在时返回 `None`
    """
    with winreg.OpenKey(key, sub_key, 0, access) as reg:
        try:
            return winreg.QueryValueEx(reg, name)[0]
        except FileNotFoundError:
            return None


def registry_delete_value(
    name: str,
    sub_key: str,
    key: RegistryRootKey | None = RegistryRootKey.CURRENT_USER,
    access: RegistryAccess | None = RegistryAccess.SET_VALUE,
) -> bool:
    """删除指定注册表键中的值

    Args:
        name (str):
            要删除的注册表值名称

        sub_key (str):
            目标注册表子键路径（不包含根键部分）

        key (RegistryRootKey | None):
            注册表根键枚举

        access (RegistryAccess | None):
            打开注册表键的访问权限

    Returns:
        bool:
            - `True`: 删除成功
            - `False`: 目标值不存在
    """
    with winreg.OpenKey(key, sub_key, 0, access) as reg:
        try:
            winreg.DeleteValue(reg, name)
            return True
        except FileNotFoundError:
            return False


def registry_enum_values(
    sub_key: str,
    key: RegistryRootKey | None = RegistryRootKey.CURRENT_USER,
    access: RegistryAccess | None = RegistryAccess.READ,
) -> dict[str, str | int | bytes | list[str] | None]:
    """枚举指定注册表键下的所有值

    Args:
        sub_key (str):
            目标注册表子键路径

        key (RegistryRootKey | None):
            注册表根键枚举

        access (RegistryAccess | None):
            打开注册表键的访问权限

    Returns:
        (dict[str, str | int | bytes | list[str] | None]):
            键为注册表值名称,
            值为对应的数据内容, 已转换为 Python 原生类型
    """
    values: dict[str, str | int | bytes | list[str] | None] = {}

    with winreg.OpenKey(key, sub_key, 0, access) as reg:
        index = 0
        while True:
            try:
                name, value, _ = winreg.EnumValue(reg, index)
                values[name] = value
                index += 1
            except OSError:
                break

    return values


def registry_set_value(
    name: str,
    value: str | int | bytes | list[str],
    reg_type: RegistryValueType,
    sub_key: str,
    key: RegistryRootKey | None = RegistryRootKey.CURRENT_USER,
    access: RegistryAccess | None = RegistryAccess.SET_VALUE,
) -> None:
    """设置或创建注册表值

    Args:
        name (str):
            注册表值名称

        value (str | int | bytes | list[str]):
            要写入的数据内容

        reg_type (RegistryValueType):
            注册表值类型

        sub_key (str):
            目标注册表子键路径

        key (RegistryRootKey | None):
            注册表根键枚举

        access (RegistryAccess | None):
            打开注册表键的访问权限
    """
    with winreg.OpenKey(key, sub_key, 0, access) as reg:
        winreg.SetValueEx(reg, name, 0, reg_type, value)


def registry_path_exists(
    sub_key: str,
    key: RegistryRootKey | None = RegistryRootKey.CURRENT_USER,
    access: RegistryAccess | None = RegistryAccess.READ,
) -> bool:
    """检查指定注册表子键路径是否存在

    Args:
        sub_key (str):
            要检查的注册表子键路径 (不包含根键)

        key (RegistryRootKey | None):
            注册表根键

        access (RegistryAccess | None):
            打开注册表时使用的访问权限

    Returns:
        bool:
            - `True`:路径存在
            - `False`: 路径不存在
    """
    try:
        with winreg.OpenKey(key, sub_key, 0, access) as _:
            return True
    except FileNotFoundError:
        return False


def registry_create_path(
    sub_key: str,
    key: RegistryRootKey | None = RegistryRootKey.CURRENT_USER,
    access: RegistryAccess | None = RegistryAccess.WRITE,
) -> None:
    """创建指定注册表子键路径 (若已存在则忽略)

    Args:
        sub_key (str):
            要创建的注册表子键路径 (不包含根键)

        key (RegistryRootKey | None):
            注册表根键

        access (RegistryAccess | None):
            打开/创建子键时的访问权限
    """
    winreg.CreateKeyEx(key, sub_key, 0, access)


def registry_delete_path(
    sub_key: str,
    key: RegistryRootKey | None = RegistryRootKey.CURRENT_USER,
) -> bool:
    """删除指定注册表子键路径（必须为空键）

    Args:
        sub_key (str):
            要删除的注册表子键路径（不包含根键）。

        key (RegistryRootKey | None):
            注册表根键，默认为 ``HKEY_CURRENT_USER``。

    Returns:
        bool:
            - `True`: 删除成功
            - `False`: 路径不存在

    Raises:
        RuntimeError: 所选的注册表路径不为空或者不能被删除
    """
    try:
        winreg.DeleteKey(key, sub_key)
        return True
    except FileNotFoundError:
        return False
    except OSError as e:
        raise RuntimeError(f"所选的注册表路径不为空或者不能被删除: {sub_key}") from e


def registry_delete_tree(
    sub_key: str,
    key: RegistryRootKey | None = RegistryRootKey.CURRENT_USER,
) -> bool:
    """递归删除注册表子键树

    Args:
        sub_key (str):
            要删除的注册表子键路径 (不包含根键)

        key (RegistryRootKey | None):
            注册表根键

    Returns:
        bool:
            - `True`: 删除成功
            - `False`: 路径不存在
    """
    try:
        with winreg.OpenKey(
            key=key,
            sub_key=sub_key,
            reserved=0,
            access=winreg.KEY_READ | winreg.KEY_WRITE,
        ) as reg:
            while True:
                try:
                    child = winreg.EnumKey(reg, 0)
                    registry_delete_tree(f"{sub_key}\\{child}", key)
                except OSError:
                    break
        winreg.DeleteKey(key, sub_key)
        return True
    except FileNotFoundError:
        return False



"""其他工具"""

import importlib.metadata
import re
import sys
from typing import (
    Annotated,
    Any,
)

import typer
from rich.console import Console
from rich.table import Table
from rich import box

from ani2xcur.config import (
    LOGGER_NAME,
    LOGGER_LEVEL,
    LOGGER_COLOR,
    IMAGE_MAGICK_WINDOWS_DOWNLOAD_URL,
    ANI2XCUR_REPOSITORY_URL,
    WIN2XCUR_REPOSITORY_URL,
    SMART_FINDER_SEARCH_DEPTH,
)
from ani2xcur.logger import get_logger
from ani2xcur.updater import self_update


logger = get_logger(
    name=LOGGER_NAME,
    level=LOGGER_LEVEL,
    color=LOGGER_COLOR,
)


def update(
    install_from_source: Annotated[
        bool,
        typer.Option(
            help="更新时是否从源码进行安装",
        ),
    ] = False,
    ani2xcur_source: Annotated[
        str | None,
        typer.Option(
            help="Ani2xcur 源仓库的 Git 链接",
        ),
    ] = None,
    win2xcur_source: Annotated[
        str | None,
        typer.Option(
            help="Win2xcur 源仓库的 Git 链接",
        ),
    ] = None,
) -> None:
    """更新 Ani2xcur"""
    try:
        self_update(
            install_from_source=install_from_source,
            ani2xcur_source=ani2xcur_source,
            win2xcur_source=win2xcur_source,
        )
    except RuntimeError as e:
        logger.error("更新 Ani2xcur 时发生错误: %s\n", e)
        logger.error("提示:")
        logger.error("1. 如果为网络问题, 可尝试配置终端代理或者 PyPI 镜像源")
        logger.error(
            "2. 如果提示`error: externally-managed-environment`, 这是由于在 Linux 中直接使用 Pip 包管理器引起的错误, "
            "可尝试使用 PIP_BREAK_SYSTEM_PACKAGES=1 ani2xcur update 忽略该错误, 或者在虚拟环境中安装 Ani2xcur"
        )
        sys.exit(1)


def version() -> None:
    """显示 Ani2xcur 和其他组件的当前版本"""

    def _display_frame(
        items: list[dict[str, Any]],
    ) -> None:
        console = Console()

        # 设置表格整体样式
        table = Table(
            header_style="bright_yellow",
            border_style="bright_black",
            box=box.ROUNDED,
        )

        # 设置列样式
        # style 参数控制该列所有单元格的默认样式
        table.add_column("组件名", style="bold white", no_wrap=True)
        table.add_column("版本", justify="left", style="white")

        for item in items:
            table.add_row(
                item["require"],
                item["version"],
            )

        console.print(table)

    requires = importlib.metadata.requires("ani2xcur")
    requires.insert(0, "ani2xcur")
    info: list[dict[str, str | None]] = []
    pkgs = [remove_optional_dependence_from_package(get_package_name(x)).split(";")[0].strip() for x in requires]
    for pkg in pkgs:
        try:
            ver = importlib.metadata.version(pkg)
        except importlib.metadata.PackageNotFoundError:
            ver = None
        info.append({"require": pkg, "version": ver})

    _display_frame(info)


def get_package_name(
    package: str,
) -> str:
    """获取 Python 软件包的包名, 去除末尾的版本声明

    Args:
        package (str): Python 软件包名
    Returns:
        str: 返回去除版本声明后的 Python 软件包名
    """
    return package.split("===")[0].split("~=")[0].split("!=")[0].split("<=")[0].split(">=")[0].split("<")[0].split(">")[0].split("==")[0].strip()


def remove_optional_dependence_from_package(
    filename: str,
) -> str:
    """移除 Python 软件包声明中可选依赖

    Args:
        filename (str): Python 软件包名
    Returns:
        str: 移除可选依赖后的软件包名, e.g. diffusers[torch]==0.10.2 -> diffusers==0.10.2
    """
    return re.sub(r"\[.*?\]", "", filename)


def env() -> None:
    """列出 Ani2xcur 使用的环境变量"""

    def _display_frame(
        items: list[dict[str, Any]],
    ) -> None:
        console = Console()

        # 设置表格整体样式
        table = Table(
            header_style="bright_yellow",
            border_style="bright_black",
            box=box.ROUNDED,
        )

        # 设置列样式
        # style 参数控制该列所有单元格的默认样式
        table.add_column("环境变量", style="bold white", no_wrap=True)
        table.add_column("值", justify="left", style="white", overflow="fold")

        for item in items:
            table.add_row(
                item["key"],
                str(item["var"]),
            )

        console.print(table)

    info = [
        {"key": "ANI2XCUR_LOGGER_NAME", "var": LOGGER_NAME},
        {"key": "ANI2XCUR_LOGGER_LEVEL", "var": LOGGER_LEVEL},
        {"key": "ANI2XCUR_LOGGER_COLOR", "var": LOGGER_COLOR},
        {"key": "IMAGE_MAGICK_WINDOWS_DOWNLOAD_URL", "var": IMAGE_MAGICK_WINDOWS_DOWNLOAD_URL},
        {"key": "ANI2XCUR_REPOSITORY_URL", "var": ANI2XCUR_REPOSITORY_URL},
        {"key": "WIN2XCUR_REPOSITORY_URL", "var": WIN2XCUR_REPOSITORY_URL},
        {"key": "ANI2XCUR_SMART_FINDER_SEARCH_DEPTH", "var": SMART_FINDER_SEARCH_DEPTH},
    ]

    _display_frame(info)



"""CLI 构建工具"""

import typer
import click


class AlphabeticalMixedGroup(typer.core.TyperGroup):
    """
    自定义的命令组
    """

    def list_commands(
        self,
        ctx: click.Context,
    ) -> list[str]:  # type: ignore[name-defined]
        """将命令按键名进行字母排序

        Args:
            ctx (click.Context): click 组件上下文
        """
        return sorted(self.commands.keys())


def typer_factory(
    help: str,  # pylint: disable=redefined-builtin
) -> typer.Typer:
    """生成 typer 装饰器

    Args:
        help (str): 命令的帮助信息
    Returns:
        typer.Typer: typer 装饰器
    """
    return typer.Typer(
        help=help,
        add_completion=True,  # 启用自动补全功能
        no_args_is_help=True,  # 不带参数运行时显示帮助信息
        cls=AlphabeticalMixedGroup,  # 使用自定义的分组类
        rich_markup_mode=None,  # 禁用 rich 标记模式
        rich_help_panel=None,  # 禁用 rich 帮助面板
        pretty_exceptions_enable=False,  # 禁用美化异常显示
    )



"""ImageMagick 管理工具"""

import os
import re
import getpass
import ctypes.util
import itertools
import platform
import shutil
from typing import Generator
from tempfile import TemporaryDirectory
from pathlib import Path
from datetime import datetime

try:
    import winreg
except ImportError:
    winreg = NotImplemented

from ani2xcur.downloader import download_file_from_url
from ani2xcur.file_operations.archive_manager import extract_archive
from ani2xcur.cmd import run_cmd
from ani2xcur.logger import get_logger
from ani2xcur.config import (
    LOGGER_COLOR,
    LOGGER_LEVEL,
    LOGGER_NAME,
    IMAGE_MAGICK_WINDOWS_DOWNLOAD_URL,
    IMAGE_MAGICK_WINDOWS_INSTALL_PATH,
)
from ani2xcur.manager.win_env_val_manager import (
    add_path_to_env_path,
    add_val_to_env,
    delete_val_from_env,
    delete_path_from_env_path,
)
from ani2xcur.utils import (
    is_admin_on_windows,
    is_root_on_linux,
)
from ani2xcur.manager.regedit import (
    RegistryAccess,
    RegistryRootKey,
    RegistryValueType,
    registry_set_value,
    registry_create_path,
    registry_delete_tree,
    registry_query_value,
)
from ani2xcur.file_operations.file_manager import remove_files
from ani2xcur.manager.desktop_config.windows import create_windows_shortcut

logger = get_logger(
    name=LOGGER_NAME,
    level=LOGGER_LEVEL,
    color=LOGGER_COLOR,
)


IMAGE_MAGICK_WINDOWS_REGISTRY_CONFIG_PATH = r"SOFTWARE\ImageMagick\Current"
"""ImageMagick 注册表配置信息"""

IMAGE_MAGICK_WINDOWS_REGISTRY_UNINSTALL_CONFIG_PATH = r"SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\ImageMagick 7.1.2 Q16-HDRI (64-bit)_is1"
"""ImageMagick 注册表卸载面板信息"""

IMAGE_MAGICK_WINDOWS_ICON_PATH = Path(os.getenv("ProgramData", "C:/ProgramData")) / "Microsoft" / "Windows" / "Start Menu" / "Programs" / "ImageMagick 7.1.2 Q16-HDRI (64-bit)"
"""ImageMagick 快捷方式路径"""


def install_image_magick_windows(
    install_path: Path | None = None,
) -> None:
    """在 Windows 系统中安装 ImageMagick

    Args:
        install_path (Path | None): 安装 ImageMagick 的目录
    Raises:
        PermissionError: 当未使用管理员权限运行时
    """
    if check_image_magick_is_installed():
        logger.info("ImageMagick 已安装在 Windows 系统中")
        return

    if not is_admin_on_windows():
        raise PermissionError("当前未使用管理员权限运行 Ani2xcur, 无法安装 ImageMagick, 请使用管理员权限进行重试")

    if install_path is None:
        install_path = IMAGE_MAGICK_WINDOWS_INSTALL_PATH

    # 注册表配置
    registry_config: list[tuple[str, str | int, RegistryValueType]] = [
        ("Version", "7.1.2", RegistryValueType.SZ),
        ("QuantumDepth", 10, RegistryValueType.DWORD),
        ("LibPath", str(install_path), RegistryValueType.SZ),
        (
            "FilterModulesPath",
            str(install_path / "modules" / "filters"),
            RegistryValueType.SZ,
        ),
        ("ConfigurePath", str(install_path), RegistryValueType.SZ),
        (
            "CoderModulesPath",
            str(install_path / "modules" / "coders"),
            RegistryValueType.SZ,
        ),
        ("BinPath", str(install_path), RegistryValueType.SZ),
    ]
    registry_sub_config: list[tuple[str, str | int, RegistryValueType]] = [
        ("LibPath", str(install_path), RegistryValueType.SZ),
        (
            "FilterModulesPath",
            str(install_path / "modules" / "filters"),
            RegistryValueType.SZ,
        ),
        ("ConfigurePath", str(install_path), RegistryValueType.SZ),
        (
            "CoderModulesPath",
            str(install_path / "modules" / "coders"),
            RegistryValueType.SZ,
        ),
        ("BinPath", str(install_path), RegistryValueType.SZ),
    ]
    uninstall_exe = str(install_path / "unins000.exe")
    uninstall_config: list[tuple[str, str | int, RegistryValueType]] = [
        ("DisplayIcon", str(install_path / "ImageMagick.ico"), RegistryValueType.SZ),
        (
            "DisplayName",
            "ImageMagick 7.1.2-12 Q16-HDRI (64-bit) (2025-12-28)",
            RegistryValueType.SZ,
        ),
        ("DisplayVersion", "7.1.2.12", RegistryValueType.SZ),
        ("EstimatedSize", int("eb6f", 16), RegistryValueType.DWORD),
        ("HelpLink", "http://www.imagemagick.org/", RegistryValueType.SZ),
        ("Inno Setup: App Path", str(install_path), RegistryValueType.SZ),
        (
            "Inno Setup: Deselected Tasks",
            "legacy_support,install_devel,install_perlmagick",
            RegistryValueType.SZ,
        ),
        (
            "Inno Setup: Icon Group",
            "ImageMagick 7.1.2 Q16-HDRI (64-bit)",
            RegistryValueType.SZ,
        ),
        ("Inno Setup: Language", "default", RegistryValueType.SZ),
        ("Inno Setup: Selected Tasks", "modifypath", RegistryValueType.SZ),
        ("Inno Setup: Setup Version", "6.2.0", RegistryValueType.SZ),
        ("Inno Setup: User", getpass.getuser(), RegistryValueType.SZ),
        ("InstallDate", datetime.now().strftime(r"%Y%m%d"), RegistryValueType.SZ),
        ("InstallLocation", str(install_path), RegistryValueType.SZ),
        ("MajorVersion", 7, RegistryValueType.DWORD),
        ("MinorVersion", 1, RegistryValueType.DWORD),
        ("NoModify", 1, RegistryValueType.DWORD),
        ("NoRepair", 1, RegistryValueType.DWORD),
        ("Publisher", "ImageMagick Studio LLC", RegistryValueType.SZ),
        ("QuietUninstallString", f'"{uninstall_exe}" /SILENT', RegistryValueType.SZ),
        ("UninstallString", uninstall_exe, RegistryValueType.SZ),
        ("URLInfoAbout", "http://www.imagemagick.org/", RegistryValueType.SZ),
        (
            "URLUpdateInfo",
            "http://www.imagemagick.org/script/download.php",
            RegistryValueType.SZ,
        ),
        ("VersionMajor", 7, RegistryValueType.DWORD),
        ("VersionMinor", 1, RegistryValueType.DWORD),
    ]

    logger.info("将 ImageMagick 安装到 Windows 系统中, 安装路径: '%s'", install_path)
    # 下载并解压 ImageMagick
    with TemporaryDirectory() as tmp_dir:
        tmp_dir = Path(tmp_dir)
        logger.debug("从 '%s' 下载 ImageMagick", IMAGE_MAGICK_WINDOWS_DOWNLOAD_URL)
        image_magick_archive_path = download_file_from_url(
            url=IMAGE_MAGICK_WINDOWS_DOWNLOAD_URL,
            save_path=tmp_dir,
        )
        extract_archive(
            archive_path=image_magick_archive_path,
            extract_to=install_path,
        )

    # 创建快捷方式
    shortcut_path = IMAGE_MAGICK_WINDOWS_ICON_PATH / "ImageMagick Web Pages.lnk"
    IMAGE_MAGICK_WINDOWS_ICON_PATH.mkdir(parents=True, exist_ok=True)
    logger.debug("为 ImageMagick 创建快捷方式, 创建路径: '%s'", shortcut_path)
    create_windows_shortcut(target_path=install_path / "index.html", shortcut_path=shortcut_path, description="ImageMagick Web Pages", working_dir=install_path)

    # 获取 ImageMagick 版本信息
    magick_bin = install_path / "magick.exe"
    version_number, quality_setting, _ = get_image_magick_version(magick_bin)
    if version_number is not None:
        version_number = version_number.split("-")[0]
    else:
        version_number = "7.1.2"
    if quality_setting is not None:
        quality_setting = re.sub(r"([A-Z]+)(\d+)", r"\1:\2", quality_setting.split("-")[0])
    else:
        quality_setting = "Q:16"

    # ImageMagick 注册表配置信息 (子表路径)
    image_magick_windows_registry_sub_config_path = rf"SOFTWARE\ImageMagick\{version_number}\{quality_setting}"

    logger.debug("写入 ImageMagick 信息到注册表中")
    # ImageMagick 配置信息
    registry_create_path(
        sub_key=IMAGE_MAGICK_WINDOWS_REGISTRY_CONFIG_PATH,
        key=RegistryRootKey.LOCAL_MACHINE,
        access=RegistryAccess.WRITE,
    )
    for key, value, dtype in registry_config:
        registry_set_value(
            name=key,
            value=value,
            reg_type=dtype,
            sub_key=IMAGE_MAGICK_WINDOWS_REGISTRY_CONFIG_PATH,
            key=RegistryRootKey.LOCAL_MACHINE,
            access=RegistryAccess.SET_VALUE,
        )

    registry_create_path(
        sub_key=image_magick_windows_registry_sub_config_path,
        key=RegistryRootKey.LOCAL_MACHINE,
        access=RegistryAccess.WRITE,
    )
    for key, value, dtype in registry_sub_config:
        registry_set_value(
            name=key,
            value=value,
            reg_type=dtype,
            sub_key=image_magick_windows_registry_sub_config_path,
            key=RegistryRootKey.LOCAL_MACHINE,
            access=RegistryAccess.SET_VALUE,
        )

    # ImageMagick 卸载配置信息
    registry_create_path(
        sub_key=IMAGE_MAGICK_WINDOWS_REGISTRY_UNINSTALL_CONFIG_PATH,
        key=RegistryRootKey.LOCAL_MACHINE,
        access=RegistryAccess.WRITE,
    )
    for key, value, dtype in uninstall_config:
        registry_set_value(
            name=key,
            value=value,
            reg_type=dtype,
            sub_key=IMAGE_MAGICK_WINDOWS_REGISTRY_UNINSTALL_CONFIG_PATH,
            key=RegistryRootKey.LOCAL_MACHINE,
            access=RegistryAccess.SET_VALUE,
        )

    # 配置环境变量
    logger.debug("为 ImageMagick 配置环境变量")
    add_image_magick_to_path(install_path)
    logger.info("ImageMagick 已安装到 Windows 系统中")


def get_image_magick_version(
    magick_bin: Path,
) -> tuple[str, str, str]:
    """获取 ImageMagick 版本号, 质量设置和架构

    Args:
        magick_bin (Path): ImageMagick 可执行文件路径
    Returns:
        (tuple[str | None, str | None, str | None]): ImageMagick 的版本号, 质量设置, 架构
    """
    try:
        result = run_cmd([magick_bin.as_posix(), "-version"], live=False)
    except RuntimeError as e:
        logger.debug("获取 ImageMagick 版本失败: %s", e)
        return None, None, None

    pattern = r"ImageMagick\s+([0-9.-]+)\s+([A-Z0-9-]+)\s+([a-zA-Z0-9]+)"
    match = re.search(pattern, result)
    if match:
        version_number = match.group(1)  # 版本号
        quality_setting = match.group(2)  # 质量设置
        architecture = match.group(3)  # 架构
    else:
        version_number = None
        quality_setting = None
        architecture = None

    logger.debug("获取 ImageMagick 的版本信息: 版本号 '%s', 质量设置 '%s', 架构 '%s'", version_number, quality_setting, architecture)
    return version_number, quality_setting, architecture


def add_image_magick_to_path(
    install_path: Path,
) -> None:
    """将 ImageMagick 添加到环境变量中

    Args:
        install_path (Path): ImageMagick 安装路径
    """
    add_path_to_env_path(
        new_path=str(install_path),
        dtype="system",
    )
    add_path_to_env_path(
        new_path=str(install_path),
        dtype="user",
    )
    add_val_to_env(name="MAGICK_HOME", value=str(install_path), dtype="system")
    add_val_to_env(
        name="MAGICK_HOME",
        value=str(install_path),
        dtype="user",
    )


def delete_image_magick_to_path(
    install_path: Path,
) -> None:
    """将 ImageMagick 从环境变量删除

    Args:
        install_path (Path): ImageMagick 安装路径
    """
    delete_path_from_env_path(key_path=str(install_path), dtype="system")
    delete_path_from_env_path(key_path=str(install_path), dtype="user")
    delete_val_from_env(name="MAGICK_HOME", dtype="system")
    delete_val_from_env(name="MAGICK_HOME", dtype="user")


def uninstall_image_magick_windows() -> None:
    """将 ImageMagick 从 Windows 系统上卸载

    Raises:
        PermissionError: 未使用管理员权限运行时
        FileNotFoundError: 当 ImageMagick 存在于系统但未找到 ImageMagick 安装路径时
        RuntimeError: 删除 ImageMagick 文件发生失败时
    """

    if not check_image_magick_is_installed():
        logger.info("ImageMagick 未安装在 Windows 系统中")
        return

    if not is_admin_on_windows():
        raise PermissionError("当前未使用管理员权限运行 Ani2xcur, 卸载安装 ImageMagick, 请使用管理员权限进行重试")

    # 查找 ImageMagick 安装路径
    install_path = find_image_magick_install_path_windows()

    if install_path is None:
        raise FileNotFoundError("未找到 ImageMagick 安装路径, 无法卸载 ImageMagick")

    logger.info("从 Windows 系统中卸载 ImageMagick 中, ImagwMagick 路径: '%s'", install_path)
    # 删除 ImageMagick 文件
    if install_path.exists():
        logger.debug("删除 ImageMagick 主文件")
        try:
            remove_files(install_path)
        except OSError as e:
            logger.error("尝试删除 ImageMagick 时发生错误: %s\n可尝试手动卸载 ImageMagick", e)
            raise RuntimeError(f"尝试删除 ImageMagick 时发生错误: {e}\n可尝试手动卸载 ImageMagick") from e

    # 删除 ImageMagick 启动图标
    if IMAGE_MAGICK_WINDOWS_ICON_PATH.exists():
        logger.debug("删除 ImageMagick 快捷方式")
        try:
            remove_files(IMAGE_MAGICK_WINDOWS_ICON_PATH)
        except OSError as e:
            logger.error("删除 ImageMagick 图标时发生错误: %s", e)
            raise RuntimeError(f"删除 ImageMagick 图标时发生错误: {e}") from e

    logger.debug("清除 ImageMagick 的注册表信息")
    # 删除注册表中的 ImageMagick 信息
    registry_delete_tree(
        sub_key=str(Path(IMAGE_MAGICK_WINDOWS_REGISTRY_CONFIG_PATH).parent),
        key=RegistryRootKey.LOCAL_MACHINE,
    )

    # 删除注册表中在卸载列表的 ImageMagick 信息
    registry_delete_tree(
        sub_key=IMAGE_MAGICK_WINDOWS_REGISTRY_UNINSTALL_CONFIG_PATH,
        key=RegistryRootKey.LOCAL_MACHINE,
    )

    logger.debug("清除 ImageMagick 的环境变量")
    # 将 ImageMagick 从环境变量中移除
    delete_image_magick_to_path(install_path)
    logger.info("从 Windows 系统卸载 ImageMagick 完成")


def find_image_magick_install_path_windows() -> Path | None:
    """在 Windows 系统中查找 ImageMagick 安装路径

    Returns:
        (Path | None): ImageMagick 安装路径, 当未找到 ImageMagick 安装路径时则返回 None
    """
    install_path = None
    for name in ["BinPath", "ConfigurePath", "LibPath"]:
        try:
            logger.debug("在 '%s' 查找 ImageMagick 的键: '%s'", IMAGE_MAGICK_WINDOWS_REGISTRY_CONFIG_PATH, name)
            install_path = registry_query_value(
                name=name,
                sub_key=IMAGE_MAGICK_WINDOWS_REGISTRY_CONFIG_PATH,
                key=RegistryRootKey.LOCAL_MACHINE,
            )
            install_path = Path(install_path)
            if not install_path.is_dir():
                install_path = None
                continue
        except FileNotFoundError:
            continue

    if install_path is None:
        try:
            logger.debug("在 '%s' 查找 ImageMagick 的键: InstallLocation", IMAGE_MAGICK_WINDOWS_REGISTRY_UNINSTALL_CONFIG_PATH)
            install_path = registry_query_value(
                name="InstallLocation",
                sub_key=IMAGE_MAGICK_WINDOWS_REGISTRY_UNINSTALL_CONFIG_PATH,
                key=RegistryRootKey.LOCAL_MACHINE,
                access=RegistryAccess.READ,
            )
            install_path = Path(install_path)
            if not install_path.is_dir():
                install_path = None
        except FileNotFoundError:
            pass

    return install_path


def install_image_magick_linux() -> None:
    """在 Linux 系统中安装 ImageMagick

    Raises:
        RuntimeError: 当 Linux 不支持自动安装 ImageMagick 时
    """
    if check_image_magick_is_installed():
        logger.info("ImageMagick 已安装在 Linux 系统中")
        return

    if not is_root_on_linux():
        raise PermissionError("当前未使用管理员权限运行 Ani2xcur, 卸载安装 ImageMagick, 请使用管理员权限进行重试")

    logger.info("安装 ImageMagick 到 Linux 系统中")
    cmd: list[list[str]] = []

    if shutil.which("apt"):
        # Debian / Ubuntu
        logger.debug("匹配到 apt 包管理器")
        cmd.append(["apt", "update"])
        cmd.append(["apt", "install", "libmagickwand-dev", "-y"])
    elif shutil.which("yum"):
        # CentOS / RHEL / Fedora
        logger.debug("匹配到 yum 包管理器")
        cmd.append(["yum", "update"])
        cmd.append(["yum", "install", "ImageMagick-devel", "-y"])
    elif shutil.which("apk"):
        # Alpine Linux
        logger.debug("匹配到 apk 包管理器")
        cmd.append(["apk", "update"])
        cmd.append(["apk", "add", "imagemagick"])
    elif shutil.which("pacman"):
        # Arch Linux
        logger.debug("匹配到 pacman 包管理器")
        cmd.append(["pacman", "-Syyu", "imagemagick", "--noconfirm"])
    elif shutil.which("zypper"):
        # openSUSE
        logger.debug("匹配到 zypper 包管理器")
        cmd.append(["zypper", "refresh"])
        cmd.append(["zypper", "install", "ImageMagick", "-y"])
    elif shutil.which("nix-env"):
        # NixOS / Nix
        logger.debug("匹配到 nix 包管理器")
        cmd.append(["nix-channel", "--update"])
        cmd.append(["nix-env", "-iA", "nixos.imagemagick"])
    else:
        raise RuntimeError("不支持的 Linux 系统, 无法自动安装 ImageMagick, 请尝试手动安装 ImageMagick")

    for c in cmd:
        run_cmd(c)

    logger.info("安装 ImageMagick 到 Linux 系统中成功")


def uninstall_image_magick_linux() -> None:
    """在 Linux 系统中卸载 ImageMagick

    Raises:
        RuntimeError: 当 Linux 不支持自动卸载 ImageMagick 时
    """
    if not check_image_magick_is_installed():
        logger.info("ImageMagick 未安装在 Linux 系统中")
        return

    if not is_root_on_linux():
        raise PermissionError("当前未使用管理员权限运行 Ani2xcur, 卸载安装 ImageMagick, 请使用管理员权限进行重试")

    logger.info("从 Linux 系统中卸载 ImageMagick 中")
    cmd: list[list[str]] = []

    if shutil.which("apt"):
        # Debian / Ubuntu
        logger.debug("匹配到 apt 包管理器")
        # 使用 purge 可以同时删除配置文件，如果只需删除程序可用 remove
        cmd.append(["apt", "purge", "libmagickwand-dev", "-y"])
        cmd.append(["apt", "autoremove", "-y"])
    elif shutil.which("yum"):
        # CentOS / RHEL / Fedora
        logger.debug("匹配到 yum 包管理器")
        cmd.append(["yum", "remove", "ImageMagick-devel", "-y"])
    elif shutil.which("apk"):
        # Alpine Linux
        logger.debug("匹配到 apk 包管理器")
        cmd.append(["apk", "del", "imagemagick"])
    elif shutil.which("pacman"):
        # Arch Linux
        logger.debug("匹配到 pacman 包管理器")
        # -Rs 会同时删除该软件及其不再被需要的依赖
        cmd.append(["pacman", "-Rs", "imagemagick", "--noconfirm"])
    elif shutil.which("zypper"):
        # openSUSE
        logger.debug("匹配到 zypper 包管理器")
        cmd.append(["zypper", "remove", "ImageMagick", "-y"])
    elif shutil.which("nix-env"):
        # NixOS / Nix
        logger.debug("匹配到 nix 包管理器")
        # 注意: nix-env 卸载时使用的是安装时的包名 (非属性路径 A)
        cmd.append(["nix-env", "-e", "imagemagick"])
    else:
        raise RuntimeError("不支持的 Linux 系统, 无法自动卸载 ImageMagick, 请尝试手动卸载")

    for c in cmd:
        run_cmd(c)

    logger.info("将 ImageMagick 从 Linux 系统中卸载成功")


def find_wand_library_paths() -> Generator[tuple[str | None, str | None], None, None]:
    """迭代尝试加载的库路径对 (Wand 库路径, Core 库路径).

    结果路径是基于启发式搜索生成的, 不一定在磁盘上真实存在.

    调用者通常需要遍历此生成器并尝试用 ctypes.CDLL 加载, 直到成功为止.

    Returns:
        (Generator[tuple[str | None, str | None], None, None]): 一个生成器, 每次迭代返回 (libwand_path, libmagick_path)
    """
    # 初始化库路径变量
    libwand = None
    libmagick = None

    # 定义可能的版本后缀, 用于匹配不同版本的 ImageMagick 库
    versions = "", "-7", "-7.Q8", "-7.Q16", "-6", "-Q16", "-Q8", "-6.Q16"

    # 定义可能的选项后缀, 如高动态范围 (HDRI) 支持
    options = "", "HDRI", "HDRI-2"

    # 获取当前操作系统类型 (Windows、Darwin/MacOS、Linux 等)
    system = platform.system()

    # 获取 MAGICK_HOME 环境变量的值, 这是 ImageMagick 的安装根目录
    magick_home = os.environ.get("MAGICK_HOME")

    # 获取自定义的库文件后缀
    magick_suffix = os.environ.get("WAND_MAGICK_LIBRARY_SUFFIX")

    if system == "Windows":
        # Windows 版的 ImageMagick 安装程序通常将编解码器和滤镜 DLL 安装在子文件夹中,
        # 我们需要将这些文件夹添加到 PATH 环境变量中, 否则稍后加载 DLL 时会失败.
        try:
            # 打开 Windows 注册表中 ImageMagick 的配置项
            with winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, r"SOFTWARE\ImageMagick\Current") as reg_key:
                # 从注册表中查询库路径、编解码器模块路径和滤镜模块路径
                libPath = winreg.QueryValueEx(reg_key, "LibPath")  # pylint: disable=invalid-name
                coderPath = winreg.QueryValueEx(reg_key, "CoderModulesPath")  # pylint: disable=invalid-name
                filterPath = winreg.QueryValueEx(reg_key, "FilterModulesPath")  # pylint: disable=invalid-name

                # 设置 magick_home 为库路径
                magick_home = libPath[0]

                # 将库路径、编解码器路径和滤镜路径添加到系统 PATH 环境变量中
                os.environ["PATH"] += str((";" + libPath[0] + ";" + coderPath[0] + ";" + filterPath[0]))
        except OSError:
            # 如果无法从注册表读取, 则使用 MAGICK_HOME 环境变量,
            # 并假设编解码器和滤镜 DLL 在相同的目录中
            pass

    # 辅助函数: 构建 ImageMagick 路径
    def magick_path(path):
        return os.path.join(magick_home, *path)

    # 生成版本和选项的所有组合
    combinations = itertools.product(versions, options)
    suffixes = []

    # 如果存在自定义后缀, 将其分割为列表
    if magick_suffix:
        suffixes = str(magick_suffix).split(";")

    # 我们需要将 combinations 生成器转换为列表, 以便可以遍历两次
    suffixes.extend(list(version + option for version, option in combinations))

    if magick_home:
        # 在调用 find_library 之前, 在 magick_home 中彻底搜索库文件
        for suffix in suffixes:
            # 在 Windows 上, API 被分成两个库. 在其他平台上, 它们全部包含在一个库中.
            if system == "Windows":
                # 尝试第一种 DLL 命名约定
                libwand = ("CORE_RL_wand_{0}.dll".format(suffix),)  # pylint: disable=consider-using-f-string
                libmagick = ("CORE_RL_magick_{0}.dll".format(suffix),)  # pylint: disable=consider-using-f-string
                yield magick_path(libwand), magick_path(libmagick)

                # 尝试第二种 DLL 命名约定
                libwand = ("CORE_RL_MagickWand_{0}.dll".format(suffix),)  # pylint: disable=consider-using-f-string
                libmagick = ("CORE_RL_MagickCore_{0}.dll".format(suffix),)  # pylint: disable=consider-using-f-string
                yield magick_path(libwand), magick_path(libmagick)

                # 尝试第三种 DLL 命名约定
                libwand = ("libMagickWand{0}.dll".format(suffix),)  # pylint: disable=consider-using-f-string
                libmagick = ("libMagickCore{0}.dll".format(suffix),)  # pylint: disable=consider-using-f-string
                yield magick_path(libwand), magick_path(libmagick)

            elif system == "Darwin":  # MacOS
                # 在 MacOS上, MagickWand 库通常也是 MagickCore 库
                libwand = (
                    "lib",
                    "libMagickWand{0}.dylib".format(suffix),  # pylint: disable=consider-using-f-string
                )
                yield magick_path(libwand), magick_path(libwand)
            else:  # Linux 和其他 Unix-like 系统
                # 尝试标准的 .so 库文件命名
                libwand = (
                    "lib",
                    "libMagickWand{0}.so".format(suffix),  # pylint: disable=consider-using-f-string
                )
                libmagick = (
                    "lib",
                    "libMagickCore{0}.so".format(suffix),  # pylint: disable=consider-using-f-string
                )
                yield magick_path(libwand), magick_path(libmagick)

                # 尝试带版本号的 .so 库文件命名 (版本 9)
                libwand = (
                    "lib",
                    "libMagickWand{0}.so.9".format(suffix),  # pylint: disable=consider-using-f-string
                )
                libmagick = (
                    "lib",
                    "libMagickCore{0}.so.9".format(suffix),  # pylint: disable=consider-using-f-string
                )
                yield magick_path(libwand), magick_path(libmagick)

                # 尝试带版本号的 .so 库文件命名 (版本6)
                libwand = (
                    "lib",
                    "libMagickWand{0}.so.6".format(suffix),  # pylint: disable=consider-using-f-string
                )
                libmagick = (
                    "lib",
                    "libMagickCore{0}.so.6".format(suffix),  # pylint: disable=consider-using-f-string
                )
                yield magick_path(libwand), magick_path(libmagick)

    # 如果在 magick_home 中未找到库文件, 使用系统默认路径搜索
    for suffix in suffixes:
        if system == "Windows":
            # 在 Windows 上分别查找 wand 和 magick 库
            libwand = ctypes.util.find_library("CORE_RL_wand_" + suffix)
            libmagick = ctypes.util.find_library("CORE_RL_magick_" + suffix)
            yield libwand, libmagick

            libwand = ctypes.util.find_library("CORE_RL_MagickWand_" + suffix)
            libmagick = ctypes.util.find_library("CORE_RL_MagickCore_" + suffix)
            yield libwand, libmagick

            libwand = ctypes.util.find_library("libMagickWand" + suffix)
            libmagick = ctypes.util.find_library("libMagickCore" + suffix)
            yield libwand, libmagick
        else:
            # 在非 Windows 系统上查找 MagickCore 和 MagickWand 库
            libmagick = ctypes.util.find_library("MagickCore" + suffix)
            libwand = ctypes.util.find_library("MagickWand" + suffix)

            # 如果 MagickCore 库存在, 则返回 wand 和 magick 路径对
            if libmagick is not None:
                yield libwand, libmagick

            # 也返回 wand 库路径, 它可能与 magick 库相同
            yield libwand, libwand


def check_image_magick_is_installed() -> bool:
    """检测 ImageMagick 是否已经安装

    Returns:
        bool: 当已经安装时则返回 True
    """
    # wand.api 在初始化时会调用 load_library() 加载 ImageMagick 的链接库
    # 当加载失败时将引发 ImportError
    # Windows 查找 ImageMagick 主要根据 MAGICK_HOME 环境变量
    # 或者注册表中`计算机\HKEY_LOCAL_MACHINE\SOFTWARE\ImageMagick\Current`的 LibPath, CoderModulesPath, FilterModulesPath
    # Linux 中是通过 ctypes.util.find_library() 查找
    for libwand_path, libmagick_path in find_wand_library_paths():
        if libwand_path is not None or libmagick_path is not None:
            logger.debug("找到 ImageMagick 库路径: ('%s', '%s')", libwand_path, libmagick_path)
            return True
        logger.debug("未找到 ImageMagick 库路径")
    return False



"""GTK xsettings 环境配置工具"""

from pathlib import Path

from ani2xcur.utils import safe_convert_to_int


GTK_XSETTINGS_PATH = Path("~/.config/xsettingsd/xsettingsd.conf").expanduser()
"""GTK xsettings 配置文件路径"""


def read_gtk_xsettings_config(
    config_path: Path,
) -> dict[str, str | int]:
    """读取 GTK xsettings 配置文件并返回配置字典

    Args:
        config_path (Path): GTK xsettings 配置文件路径
    Returns:
        (dict[str, str | int]): 配置字典
    """
    config: dict[str, str | int] = {}

    with open(config_path, "r", encoding="utf-8") as file:
        for line in file:
            line = line.strip()
            # 跳过注释行和空行
            if line.startswith("#") or not line:
                continue

            # 分割键和值
            parts = line.split(maxsplit=1)  # 只分割第一个空格
            if len(parts) != 2:
                continue

            key = parts[0].strip()
            value = parts[1].strip()

            # 移除值两端的引号（如果有的话）
            if value.startswith('"') and value.endswith('"'):
                value = value[1:-1]
            elif value.startswith("'") and value.endswith("'"):
                value = value[1:-1]

            config[key] = value

    return config


def write_gtk_xsettings_config(
    config_path: Path,
    config: dict[str, str | int],
) -> None:
    """将配置字典写入 GTK xsettings 配置文件

    Args:
        config_path (Path): GTK xsettings 配置文件路径
        config (dict[str, str | int]): 要写入的配置字典
    """
    with open(config_path, "w", encoding="utf-8") as file:
        for key, value in config.items():
            # 根据值类型决定是否加引号（字符串通常需要加引号）
            if isinstance(value, str):
                file.write(f'{key} "{value}"\n')
            else:
                file.write(f"{key} {value}\n")


def get_gtk_xsettings_cursor_theme() -> str | None:
    """获取 GTK xsettings 配置文件中的鼠标指针主题

    Returns:
        (str | None): 当前使用的鼠标指针主题名称
    """
    if not GTK_XSETTINGS_PATH.is_file():
        return None

    config = read_gtk_xsettings_config(GTK_XSETTINGS_PATH)
    return config.get("Gtk/CursorThemeName")


def get_gtk_xsettings_cursor_size() -> int | None:
    """获取 GTK xsettings 配置文件中的鼠标指针大小

    Returns:
        (int | None): 当前使用的鼠标指针大小
    """
    if not GTK_XSETTINGS_PATH.is_file():
        return None

    config = read_gtk_xsettings_config(GTK_XSETTINGS_PATH)
    size = config.get("Gtk/CursorThemeSize")
    return safe_convert_to_int(size)


def set_gtk_xsettings_cursor_theme(
    cursor_name: str,
) -> None:
    """设置 GTK xsettings 配置文件中的鼠标指针主题

    Args:
        theme_name (str): 鼠标指针主题名称
    """
    GTK_XSETTINGS_PATH.parent.mkdir(parents=True, exist_ok=True)
    if GTK_XSETTINGS_PATH.is_file():
        config = read_gtk_xsettings_config(GTK_XSETTINGS_PATH)
    else:
        config = {}

    config["Gtk/CursorThemeName"] = cursor_name
    write_gtk_xsettings_config(GTK_XSETTINGS_PATH, config)


def set_gtk_xsettings_cursor_size(
    cursor_size: int,
) -> None:
    """设置 GTK xsettings 配置文件中的鼠标指针大小

    Args:
        size (int): 鼠标指针大小
    """
    GTK_XSETTINGS_PATH.parent.mkdir(parents=True, exist_ok=True)
    if GTK_XSETTINGS_PATH.is_file():
        config = read_gtk_xsettings_config(GTK_XSETTINGS_PATH)
    else:
        config = {}

    config["Gtk/CursorThemeSize"] = str(cursor_size)
    write_gtk_xsettings_config(GTK_XSETTINGS_PATH, config)



"""系统鼠标指针管理工具"""

import sys
import traceback
from typing import (
    Annotated,
    Any,
)
from pathlib import Path
from tempfile import TemporaryDirectory

import typer
from rich.console import Console
from rich.table import Table
from rich import box

from ani2xcur.manager.base import (
    WINDOWS_USER_CURSOR_PATH,
    LINUX_USER_ICONS_PATH,
)
from ani2xcur.manager.win_cur_manager import (
    install_windows_cursor,
    delete_windows_cursor,
    export_windows_cursor,
    set_windows_cursor_theme,
    set_windows_cursor_size,
    list_windows_cursors,
    get_windows_cursor_info,
)
from ani2xcur.manager.linux_cur_manager import (
    install_linux_cursor,
    delete_linux_cursor,
    export_linux_cursor,
    set_linux_cursor_theme,
    set_linux_cursor_size,
    list_linux_cursors,
    get_linux_cursor_info,
)
from ani2xcur.config import (
    LOGGER_NAME,
    LOGGER_LEVEL,
    LOGGER_COLOR,
    SMART_FINDER_SEARCH_DEPTH,
)
from ani2xcur.logger import get_logger
from ani2xcur.smart_finder import (
    find_inf_file,
    find_desktop_entry_file,
)

logger = get_logger(
    name=LOGGER_NAME,
    level=LOGGER_LEVEL,
    color=LOGGER_COLOR,
)


def install_cursor(
    input_path: Annotated[
        Path,
        typer.Argument(
            help="鼠标指针文件的路径, 输入鼠标指针压缩包文件路径, 或者: Windows 平台中输入 inf / ani / cur 文件路径; Linux 平台输入 index.theme 文件路径",
            resolve_path=True,
        ),
    ],
    install_path: Annotated[
        Path | None,
        typer.Option(
            help="自定义鼠标指针文件安装路径, 默认为鼠标指针配置文件中指定的安装路径",
            resolve_path=True,
        ),
    ] = None,
    use_inf_config_path: Annotated[
        bool,
        typer.Option(
            help="(仅 Windows 平台) 使用 INF 配置文件中的鼠标指针安装路径",
        ),
    ] = False,
) -> None:
    """将鼠标指针安装到系统中"""
    if sys.platform == "win32":
        if install_path is None:
            if use_inf_config_path:
                logger.info("使用 INF 配置文件中的鼠标指针安装路径")
            else:
                install_path = WINDOWS_USER_CURSOR_PATH
                logger.info("未指定鼠标指针安装路径, 使用默认鼠标指针安装路径: '%s'", install_path)
        else:
            logger.info("使用自定义鼠标指针安装路径: '%s'", install_path)

        logger.info("安装的 Windows 鼠标指针: '%s'", input_path)
        with TemporaryDirectory() as temp_dir:
            temp_dir = Path(temp_dir)
            inf_file = find_inf_file(
                input_file=input_path,
                temp_dir=temp_dir,
                depth=SMART_FINDER_SEARCH_DEPTH,
            )
            if inf_file is None:
                logger.error("未找到鼠标指针的 INF 配置文件路径, 该鼠标指针文件无法安装")
                sys.exit(1)

            try:
                install_windows_cursor(
                    inf_file=inf_file,
                    cursor_install_path=install_path,
                )
            except PermissionError as e:
                traceback.print_exc()
                logger.error(
                    "在 Windows 系统上安装鼠标指针时发生错误: '%s'\n请检查是否使用管理员权限运行 Ani2xcur 运行, 或者尝试使用 --install-path 参数指定其他鼠标指针的安装路径", e
                )
                sys.exit(1)
    elif sys.platform == "linux":
        if install_path is None:
            install_path = LINUX_USER_ICONS_PATH
            logger.info("未指定鼠标指针安装路径, 使用默认鼠标指针安装路径: '%s'", install_path)
        else:
            logger.info("使用自定义鼠标指针安装路径: '%s'", install_path)

        logger.info("安装的 Linux 鼠标指针: '%s'", input_path)
        with TemporaryDirectory() as temp_dir:
            temp_dir = Path(temp_dir)
            desktop_entry_file = find_desktop_entry_file(
                input_file=input_path,
                temp_dir=temp_dir,
                depth=SMART_FINDER_SEARCH_DEPTH,
            )

            if desktop_entry_file is None:
                logger.error("未找到鼠标指针的 DesktopEntry 配置文件路径")
                sys.exit(1)

            if not (desktop_entry_file.parent / "cursors").is_dir():
                logger.error("鼠标指针目录缺失, 无法进行鼠标指针转换")
                sys.exit(1)

            try:
                install_linux_cursor(
                    desktop_entry_file=desktop_entry_file,
                    cursor_install_path=install_path,
                )
            except (FileNotFoundError, RuntimeError) as e:
                traceback.print_exc()
                logger.error(
                    "在 Windows 系统上安装鼠标指针时发生错误: %s\n请检查是否使用管理员权限运行 Ani2xcur 运行, 或者尝试使用 --install-path 参数指定其他鼠标指针的安装路径\n鼠标指针文件可能也出现损坏, 也请检查鼠标指针文件的完整性",
                    e,
                )
                sys.exit(1)
    else:
        logger.error("不支持的系统: %s", sys.platform)
        sys.exit(1)


def uninstall_cursor(
    cursor_name: Annotated[
        str,
        typer.Argument(
            help="要删除的鼠标指针名称",
        ),
    ],
) -> None:
    """删除系统中指定的鼠标指针"""
    if sys.platform == "win32":
        try:
            delete_windows_cursor(cursor_name)
        except RuntimeError as e:
            traceback.print_exc()
            logger.error("删除鼠标指针时发生错误: %s\n可能为没有权限进行删除鼠标指针文件, 可尝试使用管理员权限运行 Ani2xcur", e)
            sys.exit(1)
        except ValueError as e:
            traceback.print_exc()
            logger.error("删除鼠标指针时发生错误: %s\n请检查要删除的鼠标指针是否存在或者正在使用", e)
            sys.exit(1)
    elif sys.platform == "linux":
        try:
            delete_linux_cursor(cursor_name)
        except RuntimeError as e:
            traceback.print_exc()
            logger.error("删除鼠标指针时发生错误: %s\n可能为没有权限进行删除鼠标指针文件, 可尝试使用 root 权限运行 Ani2xcur", e)
            sys.exit(1)
        except ValueError as e:
            traceback.print_exc()
            logger.error("删除鼠标指针时发生错误: %s\n请检查要删除的鼠标指针是否存在", e)
            sys.exit(1)
    else:
        logger.error("不支持的系统: %s", sys.platform)
        sys.exit(1)


def export_cursor(
    cursor_name: Annotated[
        str,
        typer.Argument(
            help="要导出的鼠标指针名称",
        ),
    ],
    output_path: Annotated[
        Path,
        typer.Argument(
            help="鼠标指针的导出路径",
            resolve_path=True,
        ),
    ],
    custom_install_path: Annotated[
        Path | None,
        typer.Option(
            help="自定义鼠标指针配置文件在安装时的文件安装路径",
            resolve_path=True,
        ),
    ] = None,
) -> None:
    """将鼠标指针从系统中导出"""
    if sys.platform == "win32":
        try:
            path = export_windows_cursor(
                cursor_name=cursor_name,
                output_path=output_path,
                custom_install_path=custom_install_path,
            )
            logger.info("Windows 鼠标指针导出完成, 导出路径: '%s'", path)
        except ValueError as e:
            traceback.print_exc()
            logger.error("导出鼠标指针发生错误: %s\n请检查导出的鼠标指针是否存在于系统中", e)
            sys.exit(1)
    elif sys.platform == "linux":
        try:
            path = export_linux_cursor(
                cursor_name=cursor_name,
                output_path=output_path,
                custom_install_path=custom_install_path,
            )
            logger.info("Windows 鼠标指针导出完成, 导出路径: '%s'", path)
        except ValueError as e:
            traceback.print_exc()
            logger.error("导出鼠标指针发生错误: %s\n请检查导出的鼠标指针是否存在于系统中", e)
            sys.exit(1)
        except RuntimeError as e:
            traceback.print_exc()
            logger.error("导出鼠标指针发生错误: %s\n可能为导出路径无权限读写, 可尝试修改导出路径进行尝试, 或者使用 root 权限运行 Ani2xcur", e)
            sys.exit(1)
    else:
        logger.error("不支持的系统: %s", sys.platform)
        sys.exit(1)


def set_cursor_theme(
    cursor_name: Annotated[
        str,
        typer.Argument(
            help="要指定的鼠标指针名称",
        ),
    ],
) -> None:
    """设置系统要使用的鼠标指针主题"""
    if sys.platform == "win32":
        try:
            set_windows_cursor_theme(cursor_name)
        except ValueError as e:
            traceback.print_exc()
            logger.error("设置鼠标指针主题时发生错误: %s\n请检查要设置的鼠标指针主题是否安装到系统中", e)
            sys.exit(1)
    elif sys.platform == "linux":
        try:
            set_linux_cursor_theme(cursor_name)
        except ValueError as e:
            traceback.print_exc()
            logger.error("设置鼠标指针主题时发生错误: %s\n请检查要设置的鼠标指针主题是否安装到系统中", e)
            sys.exit(1)
    else:
        logger.error("不支持的系统: %s", sys.platform)
        sys.exit(1)


def set_cursor_size(
    cursor_size: Annotated[
        int,
        typer.Argument(
            help="要指定的鼠标指针大小",
        ),
    ],
) -> None:
    """设置系统要使用的鼠标指针大小"""
    if sys.platform == "win32":
        try:
            set_windows_cursor_size(cursor_size)
        except TypeError as e:
            traceback.print_exc()
            logger.error("设置鼠标指针大小时发生错误: %s\n请检查鼠标指针大小的是否为整数", e)
            sys.exit(1)
        except ValueError as e:
            traceback.print_exc()
            logger.error("设置鼠标指针大小时发生错误: %s\n请检查鼠标指针大小的值是否在合法范围", e)
            sys.exit(1)
    elif sys.platform == "linux":
        try:
            set_linux_cursor_size(cursor_size)
        except TypeError as e:
            traceback.print_exc()
            logger.error("设置鼠标指针大小时发生错误: %s\n请检查鼠标指针大小的是否为整数", e)
            sys.exit(1)
        except ValueError as e:
            traceback.print_exc()
            logger.error("设置鼠标指针大小时发生错误: %s\n请检查鼠标指针大小的值是否在合法范围", e)
            sys.exit(1)
    else:
        logger.error("不支持的系统: %s", sys.platform)
        sys.exit(1)


def list_cursor() -> None:
    """列出当前系统中已安装的鼠标指针"""

    def _display_frame(
        items: list[dict[str, Any]],
    ) -> None:
        console = Console()

        # 设置表格整体样式
        table = Table(
            header_style="bright_yellow",
            border_style="bright_black",
            box=box.ROUNDED,
        )

        # 设置列样式
        # style 参数控制该列所有单元格的默认样式
        table.add_column("鼠标指针名称", style="bold white", no_wrap=True)
        table.add_column("数量", justify="left", style="white")
        table.add_column("安装路径", style="cyan", overflow="fold")

        for item in items:
            path = ", ".join([str(x) for x in item["install_paths"]])
            count = len(item["cursor_files"])

            table.add_row(item["name"], str(count), path)

        console.print(table)

    if sys.platform == "win32":
        logger.info("获取 Windows 系统中已安装的鼠标指针列表")
        cursors = list_windows_cursors()
    elif sys.platform == "linux":
        logger.info("获取 Linux 系统中已安装的鼠标指针列表")
        cursors = list_linux_cursors()
    else:
        logger.error("不支持的系统: %s", sys.platform)
        sys.exit(1)

    _display_frame(cursors)


def get_current_cursor() -> None:
    """显示当前系统中设置的鼠标指针名称和大小"""

    def _display_frame(
        items: list[dict[str, Any]],
    ) -> None:
        console = Console()

        # 设置表格整体样式
        table = Table(
            header_style="bright_yellow",
            border_style="bright_black",
            box=box.ROUNDED,
        )

        # 设置列样式
        # style 参数控制该列所有单元格的默认样式
        table.add_column("平台", style="bold white", no_wrap=True)
        table.add_column("鼠标指针主题", justify="left", style="white")
        table.add_column("鼠标指针大小", style="cyan", overflow="fold")

        for item in items:
            platform = item["platform"]
            cursor_name = item["cursor_name"]
            cursor_size = item["cursor_size"]

            table.add_row(platform, str(cursor_name), str(cursor_size))

        console.print(table)

    if sys.platform == "win32":
        info = get_windows_cursor_info()
    elif sys.platform == "linux":
        info = get_linux_cursor_info()
    else:
        logger.error("不支持的系统: %s", sys.platform)
        sys.exit(1)

    _display_frame(info)



"""鼠标指针包转换工具"""

import os
from pathlib import Path
from tempfile import TemporaryDirectory

from tqdm import tqdm

from ani2xcur.config_parse.win import dict_to_inf_strings_format
from ani2xcur.manager.base import LINUX_CURSOR_LINKS
from ani2xcur.manager.win_cur_manager import (
    extract_scheme_info_from_inf,
    generate_cursor_scheme_inf_string,
)
from ani2xcur.manager.base import CURSOR_KEYS
from ani2xcur.manager.linux_cur_manager import (
    extract_scheme_info_from_desktop_entry,
    generate_install_script,
)
from ani2xcur.config import (
    LINUX_CURSOR_SOURCE_PATH,
    LOGGER_NAME,
    LOGGER_LEVEL,
    LOGGER_COLOR,
)
from ani2xcur.logger import get_logger
from ani2xcur.cursor_conversion.win2xcur_warp import (
    win2xcur_process,
    x2wincur_process,
    Win2xcurArgs,
    X2wincurArgs,
)
from ani2xcur.file_operations.file_manager import (
    copy_files,
    save_create_symlink,
)

logger = get_logger(
    name=LOGGER_NAME,
    level=LOGGER_LEVEL,
    color=LOGGER_COLOR,
)


def win_cursor_to_x11(
    inf_file: Path,
    output_path: Path,
    win2x_args: Win2xcurArgs,
) -> Path:
    """将 Windows 鼠标指针包转换为 Linux 的鼠标指针包

    Args:
        inf_file (Path): Windows 鼠标指针包中的 INF 文件路径
        output_path (Path): 导出路径
        win2x_args (Win2xcurArgs): 传递给 win2xcur_process() 函数的参数
    Returns:
        Path: Linux 的鼠标指针包的完整路径
    """
    win_scheme = extract_scheme_info_from_inf(inf_file)
    logger.debug("INF 鼠标指针配置文件内容: %s", win_scheme)
    cursor_map = win_scheme["cursor_map"]
    cursor_name = win_scheme["scheme_name"]
    win2x_path_list: list[tuple[str, Path, Path]] = []
    completed_cursor_list: list[tuple[str, Path]] = []
    link_file_list: list[tuple[Path, Path]] = []

    with TemporaryDirectory() as tmp_dir:
        tmp_dir = Path(tmp_dir)

        # 创建 cursors 文件夹用于存放鼠标指针
        cursors_dir = tmp_dir / cursor_name / "cursors"
        cursors_dir.mkdir(parents=True, exist_ok=True)

        logger.info("配置 '%s' 鼠标指针的转换参数", cursor_name)

        # 生成要进行鼠标指针的转换列表
        for win, linux in zip(CURSOR_KEYS["win"], CURSOR_KEYS["linux"]):
            src = cursor_map[win]["src_path"]
            dst = cursors_dir / linux
            if src is None:
                # 使用补全文件
                src = LINUX_CURSOR_SOURCE_PATH / linux
                logger.debug("添加补全列表: Windows 鼠标指针 `%s` -> Linux 鼠标指针 '%s'", src, dst)
                completed_cursor_list.append((src, dst))
                continue

            logger.debug("添加转换列表: Windows 鼠标指针 `%s` -> Linux 鼠标指针 '%s'", src, dst)
            win2x_path_list.append((linux, src, dst))

        # 补全文件列表
        completed_cursor_list.append((LINUX_CURSOR_SOURCE_PATH / "vertical-text", cursors_dir / "vertical-text"))
        completed_cursor_list.append(
            (
                LINUX_CURSOR_SOURCE_PATH / "wayland-cursor",
                cursors_dir / "wayland-cursor",
            )
        )
        completed_cursor_list.append((LINUX_CURSOR_SOURCE_PATH / "zoom-out", cursors_dir / "zoom-out"))
        completed_cursor_list.append((LINUX_CURSOR_SOURCE_PATH / "zoom-in", cursors_dir / "zoom-in"))

        # 链接文件列表
        link_file_list = [(Path(s), Path(v)) for s, v in LINUX_CURSOR_LINKS]

        # 转换鼠标指针文件
        logger.debug("要进行转换的鼠标指针列表: %s", win2x_path_list)
        for name, src, dst in tqdm(win2x_path_list, desc="转换鼠标指针文件"):
            win2x_args["input_file"] = src
            win2x_args["output_path"] = cursors_dir
            win2x_args["save_name"] = name
            logger.debug("调用 win2xcur 使用的参数: %s", win2x_args)
            win2xcur_process(**win2x_args)

        # 补全鼠标指针文件
        logger.debug("要进行补全的鼠标指针列表: %s", completed_cursor_list)
        for src, dst in tqdm(completed_cursor_list, desc="补全鼠标指针文件"):
            copy_files(src, dst)

        # 创建链接文件
        current_path = Path().absolute()
        os.chdir(cursors_dir)
        logger.debug("要进行链接的鼠标指针别名: %s", link_file_list)
        for s, v in tqdm(link_file_list, desc="链接鼠标指针别名"):
            save_create_symlink(s, v)

        os.chdir(current_path)

        # 创建配置文件
        generate_linux_cursor_config(
            cursor_name=cursor_name,
            cursor_path=tmp_dir / cursor_name,
        )

        # 创建安装脚本
        generate_install_script(
            cursor_name=cursor_name,
            save_dir=tmp_dir / cursor_name,
        )

        # 导出文件到输出文件夹
        copy_files((tmp_dir / cursor_name), output_path)

    return output_path / cursor_name


def generate_linux_cursor_config(
    cursor_name: str,
    cursor_path: Path,
) -> None:
    """生成鼠标指针配置文件

    Args:
        cursor_name (str): 鼠标指针名称
        cursor_path (Path): 配置文件的保存路径
    """
    cursor_config = f"""
[Icon Theme]
Name={cursor_name}
Inherits={cursor_name}
""".strip()
    index_config = f"""
[Icon Theme]
Name={cursor_name}
Comment={cursor_name} cursor for Linux
Inherits={cursor_name}
""".strip()

    logger.debug("鼠标指针配置文件内容:\n\n- cursor.theme:\n````\n%s\n```\n\n- index.theme:\n```\n%s\n```", cursor_config, index_config)
    with open((cursor_path / "cursor.theme"), "w", encoding="utf-8") as file:
        file.write(cursor_config)
    with open((cursor_path / "index.theme"), "w", encoding="utf-8") as file:
        file.write(index_config)


def x11_cursor_to_win(
    desktop_entry_file: Path,
    output_path: Path,
    x2win_args: X2wincurArgs,
) -> Path:
    """将 Linux 鼠标指针包转换为 Windows 的鼠标指针包

    Args:
        desktop_entry_file (Path): Linux 鼠标指针包中的 DesktopEntry 文件路径
        output_path (Path): 导出路径
        x2win_args (X2wincurArgs): 传递给 x2wincur_process() 函数的参数
    Returns:
        Path: Linux 的鼠标指针包的完整路径
    """
    linux_scheme = extract_scheme_info_from_desktop_entry(desktop_entry_file)
    logger.debug("DesktopEntry 鼠标指针配置文件内容: %s", linux_scheme)
    cursor_map = linux_scheme["cursor_map"]
    cursor_name = linux_scheme["scheme_name"]
    x2win_path_list: list[tuple[str, Path, Path]] = []
    cursor_save_paths: list[tuple[str, Path | None]] = []

    with TemporaryDirectory() as tmp_dir:
        tmp_dir = Path(tmp_dir)

        # 创建文件夹用于存放鼠标指针
        cursors_dir = tmp_dir / cursor_name
        cursors_dir.mkdir(parents=True, exist_ok=True)

        logger.info("配置 '%s' 鼠标指针的转换参数", cursor_name)

        # 生成要进行鼠标指针的转换列表
        for win, linux in zip(CURSOR_KEYS["win"], CURSOR_KEYS["linux"]):
            src = cursor_map[win]["src_path"]
            dst = cursors_dir / linux
            if src is None:
                # 使用补全文件
                src = LINUX_CURSOR_SOURCE_PATH / linux

            x2win_path_list.append((win, src, dst))

        # 转换鼠标指针文件
        logger.debug("要进行转换的鼠标指针列表: %s", x2win_path_list)
        for name, src, dst in tqdm(x2win_path_list, desc="转换鼠标指针文件"):
            x2win_args["input_file"] = src
            x2win_args["output_path"] = cursors_dir
            x2win_args["save_name"] = name
            logger.debug("调用 x2wincur 的参数: %s", x2win_args)
            if src is not None:
                cursor_save_path = x2wincur_process(**x2win_args)
            else:
                cursor_save_path = None

            logger.debug("添加转换列表: Linux 鼠标指针 `%s` -> Windows 鼠标指针 '%s'", src, dst)
            cursor_save_paths.append((name, cursor_save_path))

        # 创建配置文件
        generate_win_cursor_config(
            cursor_name=cursor_name,
            cursor_path=cursors_dir,
            cursor_save_paths=cursor_save_paths,
        )

        save_dir = output_path / cursor_name

        # 导出文件到输出文件夹
        copy_files((tmp_dir / cursor_name), save_dir)

    return save_dir


def generate_win_cursor_config(
    cursor_name: str,
    cursor_path: Path,
    cursor_save_paths: list[tuple[str, Path | None]],
) -> None:
    """生成 Windows 鼠标指针配置文件

    Args:
        cursor_name (str): 鼠标指针名称
        cursor_path (Path): 鼠标指针包路径
        cursor_save_paths (list[tuple[str, Path | None]]): 鼠标指针类型对应的保存路径
    """

    cursor_paths: list[Path] = []  # 用于导出的路径列表
    strings: dict[str, str] = {}  # [Strings] 部分
    wreg_list: list[str] = []  # [Wreg] 部分
    paths_to_reg: list[str] = []  # [Scheme.Reg] 部分

    destination_dirs = r'10,"%CUR_DIR%"'
    strings["SCHEME_NAME"] = cursor_name
    strings["CUR_DIR"] = rf"Cursors\{cursor_name}"
    wreg_list.append(r'HKCU,"Control Panel\Cursors",,0x00020000,"%SCHEME_NAME%"')

    for name, path in cursor_save_paths:
        if path is not None:
            cursor_paths.append(path)
            wreg_list.append(rf'HKCU,"Control Panel\Cursors",{name},0x00020000,"%10%\%CUR_DIR%\%{name}%"')
            strings[name] = path.name
            paths_to_reg.append(rf"%10%\%CUR_DIR%\%{name}%")
        else:
            paths_to_reg.append("")

    # 配置 [Wreg] 字段
    wreg_list.append(r'HKLM,"SOFTWARE\Microsoft\Windows\CurrentVersion\Runonce\Setup\","",,"rundll32.exe shell32.dll,Control_RunDLL main.cpl @0"')
    wreg = "\n".join(wreg_list)

    # 配置 [Scheme.Reg] 字段
    paths_to_reg_string = ",".join(paths_to_reg)
    scheme_reg = rf'HKCU,"Control Panel\Cursors\Schemes","%SCHEME_NAME%",0x00020000,"{paths_to_reg_string}"'

    # 配置 [Scheme.Cur] 字段
    scheme_cur = "\n".join([f'"{x.name}"' for x in cursor_paths])

    inf = generate_cursor_scheme_inf_string(
        destination_dirs=destination_dirs,
        wreg=wreg,
        scheme_reg=scheme_reg,
        scheme_cur=scheme_cur,
        strings=dict_to_inf_strings_format(strings),
    )

    logger.debug("鼠标指针配置文件内容:\n```\n%s\n```", inf)
    with open(cursor_path / "AutoSetup.inf", "w", encoding="gbk") as f:
        f.write(inf)



"""Shell 命令执行器

不同平台下 subprocess 的执行结果不一致, 以下为不同平台的测试结果:
```
Test Platform: linux
Case 1: {'cmd': ['/usr/bin/git', '--version'], 'shell': False} -> Success
Case 2: {'cmd': ['/usr/bin/git', '--version'], 'shell': True} -> Failed
Case 3: {'cmd': '"/usr/bin/git" --version', 'shell': False} -> Failed
Case 4: {'cmd': '"/usr/bin/git" --version', 'shell': True} -> Success
Case 5: {'cmd': ['/usr/bin/git', '--version'], 'shell': False} -> Success
Case 6: {'cmd': ['/usr/bin/git', '--version'], 'shell': True} -> Failed
Case 7: {'cmd': '"/usr/bin/git" --version', 'shell': False} -> Failed
Case 8: {'cmd': '"/usr/bin/git" --version', 'shell': True} -> Success

Test Platform: win32
Case 1: {'cmd': ['C:\\Program Files\\Git\\mingw64\\bin\\git.EXE', '--version'], 'shell': False} -> Success
Case 2: {'cmd': ['C:\\Program Files\\Git\\mingw64\\bin\\git.EXE', '--version'], 'shell': True} -> Success
Case 3: {'cmd': '"C:\\Program Files\\Git\\mingw64\\bin\\git.EXE" --version', 'shell': False} -> Success
Case 4: {'cmd': '"C:\\Program Files\\Git\\mingw64\\bin\\git.EXE" --version', 'shell': True} -> Success
Case 5: {'cmd': ['C:/Program Files/Git/mingw64/bin/git.EXE', '--version'], 'shell': False} -> Success
Case 6: {'cmd': ['C:/Program Files/Git/mingw64/bin/git.EXE', '--version'], 'shell': True} -> Success
Case 7: {'cmd': '"C:/Program Files/Git/mingw64/bin/git.EXE" --version', 'shell': False} -> Success
Case 8: {'cmd': '"C:/Program Files/Git/mingw64/bin/git.EXE" --version', 'shell': True} -> Success

Test Platform: darwin
Case 1: {'cmd': ['/opt/homebrew/bin/git', '--version'], 'shell': False} -> Success
Case 2: {'cmd': ['/opt/homebrew/bin/git', '--version'], 'shell': True} -> Failed
Case 3: {'cmd': '"/opt/homebrew/bin/git" --version', 'shell': False} -> Failed
Case 4: {'cmd': '"/opt/homebrew/bin/git" --version', 'shell': True} -> Success
Case 5: {'cmd': ['/opt/homebrew/bin/git', '--version'], 'shell': False} -> Success
Case 6: {'cmd': ['/opt/homebrew/bin/git', '--version'], 'shell': True} -> Failed
Case 7: {'cmd': '"/opt/homebrew/bin/git" --version', 'shell': False} -> Failed
Case 8: {'cmd': '"/opt/homebrew/bin/git" --version', 'shell': True} -> Success
```

- 对于 Linux 平台, 当使用 Shell=True 时, 应使用字符串命令; Shell=False 时, 使用列表命令
- 对于 Windows 平台, 当使用 Shell=True / Shell=False 时, 使用字符串命令和列表命令都可行
- 对于 MacOS 平台, 当使用 Shell=True 时, 应使用字符串命令; Shell=False 时, 使用列表命令 (行为和 Linux 中的一致)
"""

import os
import sys
import shlex
import subprocess
from pathlib import Path

from ani2xcur.logger import get_logger
from ani2xcur.config import (
    LOGGER_COLOR,
    LOGGER_LEVEL,
    LOGGER_NAME,
)


logger = get_logger(
    name=LOGGER_NAME,
    level=LOGGER_LEVEL,
    color=LOGGER_COLOR,
)


def preprocess_command(
    command: list[str] | str,
    shell: bool,
) -> list[str] | str:
    """针对不同平台对命令进行预处理

    Args:
        command (list[str] | str): 原始命令
        shell (bool): 是否调用 Shell
    Returns:
        (list[str] | str): 处理后的命令
    """
    if sys.platform == "win32":
        # Windows
        # 字符串命令和列表命令都可行
        return command
    else:
        # Linux / MacOS
        if shell:
            # 使用字符串命令
            if isinstance(command, list):
                return shlex.join(command)
            return command
        # 使用列表命令
        if isinstance(command, str):
            return shlex.split(command)
        return command


def run_cmd(
    command: str | list[str],
    custom_env: dict[str, str] | None = None,
    live: bool | None = True,
    shell: bool | None = None,
    cwd: Path | None = None,
    check: bool | None = True,
) -> str | None:
    """执行 Shell 命令

    Args:
        command (str | list[str]): 要执行的命令
        custom_env (dict[str, str] | None): 自定义环境变量
        live (bool | None): 是否实时输出命令执行日志
        shell (bool | None): 是否使用内置 Shell 执行命令
        cwd (Path | str | None): 执行进程时的起始路径
        check (bool | None): 检查进程退出状态, 当异常退出时引发 RuntimeError
    Returns:
        (str | None): 命令执行时输出的内容, 当 live=True / 执行命令发生错误时输出内容为 None
    Raises:
        RuntimeError: 当命令执行失败时
    """

    if shell is None:
        shell = sys.platform != "win32"

    if custom_env is None:
        custom_env = os.environ.copy()

    command_to_exec = preprocess_command(command=command, shell=shell)

    kwargs = {
        "args": command_to_exec,
        "shell": shell,
        "env": custom_env,
        "cwd": cwd,
        "encoding": "utf-8",
        "errors": "ignore",
    }

    if not live:
        kwargs["stdout"] = kwargs["stderr"] = subprocess.PIPE

    logger.debug("执行命令的参数: %s", kwargs)

    result: subprocess.CompletedProcess[str] = subprocess.run(**kwargs)  # pylint: disable=subprocess-run-check

    if check and result.returncode != 0:
        errors = [
            "执行命令时发生错误",
            f"命令: {command_to_exec}",
            f"错误代码: {result.returncode}",
        ]
        if result.stdout:
            errors.append(f"标准输出: {result.stdout}")
        if result.stderr:
            errors.append(f"错误输出: {result.stderr}")

        raise RuntimeError("\n".join(errors))

    return result.stdout



"""鼠标指针配置文件智能搜索"""

from pathlib import Path

from ani2xcur.config_parse.win import parse_inf_file_content
from ani2xcur.config_parse.linux import parse_desktop_entry_content
from ani2xcur.file_operations.archive_manager import (
    is_supported_archive_format,
    extract_archive,
)
from ani2xcur.file_operations.file_manager import get_file_list
from ani2xcur.utils import (
    generate_random_string,
    is_http_or_https,
)
from ani2xcur.downloader import download_file_from_url
from ani2xcur.logger import get_logger
from ani2xcur.config import (
    LOGGER_COLOR,
    LOGGER_LEVEL,
    LOGGER_NAME,
)

logger = get_logger(
    name=LOGGER_NAME,
    level=LOGGER_LEVEL,
    color=LOGGER_COLOR,
)


def find_desktop_entry_file(
    input_file: Path | str,
    temp_dir: Path,
    depth: int | None = 0,
    visited: set[Path] | None = None,
    is_toplevel: bool = True,  # pylint: disable=unused-argument
) -> Path | None:
    """搜索 DesktopEntry 文件路径

    Args:
        input_file (Path | str): 任意文件路径或压缩包的下载链接
        temp_dir (Path): 临时文件夹, 用于保存寻找文件时产生的临时文件
        depth (int | None): 递归搜索文件的深度
        visited (set[Path] | None): 已访问路径集合, 用于防止死循环
        is_toplevel (bool): 是否为顶层调用
    Returns:
        (Path | None): 当找到 DesktopEntry 文件时则返回其路径, 否则返回 None
    """

    # 初始化去重集合
    if visited is None:
        visited = set()

    logger.debug("已搜索路径: '%s'", visited)
    logger.debug("查找路径: '%s'", input_file)

    # 预处理: 如果是字符串路径且不是 URL, 先转为 Path
    # 这样可以确保 visited 集合中的 Path 都是 resolve 过的, 避免字符串路径绕过去重
    is_url = is_http_or_https(input_file)

    # 统一去重检查 (针对 URL 或 Path)
    if input_file in visited:
        return None

    # 递归深度控制
    if depth < 0:
        return None

    # 处理 URL 情况
    if is_url:
        visited.add(input_file)
        try:
            download_file = download_file_from_url(
                url=input_file,
                save_path=temp_dir,
            )
            if not is_supported_archive_format(download_file):
                logger.debug("下载的文件不是支持的压缩包格式: '%s'", download_file)
                return None

            logger.debug("从 '%s' 下载成功, 准备解压搜索: '%s'", input_file, download_file)
            extract_path = temp_dir / generate_random_string()
            extract_archive(
                archive_path=download_file,
                extract_to=extract_path,
            )
            # 递归调用 (标记 is_toplevel=False)
            return find_desktop_entry_file(
                input_file=extract_path,
                temp_dir=temp_dir,
                depth=depth - 1,  # 避免压缩包嵌套造成死循环
                visited=visited,
                is_toplevel=False,
            )
        except Exception as e:
            logger.debug("从 '%s' 下载文件时发生错误, 终止寻找: %s", input_file, e)
            return None

    # 文件系统逻辑处理
    try:
        # 确保 input_file 是 Path 对象
        abs_path = Path(input_file).resolve()
    except Exception:
        return None

    if not abs_path.exists() or abs_path in visited:
        return None
    visited.add(abs_path)

    # 验证 DesktopEntry 文件完整性
    if abs_path.is_file() and abs_path.name.lower().endswith(".theme"):
        try:
            _ = parse_desktop_entry_content(abs_path)
            logger.debug("搜索到 DesktopEntry 文件路径: '%s'", abs_path)
            return abs_path
        except ValueError:
            return None

    # 文件为压缩包时则尝试解压并遍历解压的文件夹
    if is_supported_archive_format(abs_path):
        logger.debug("检测到压缩包，准备解压: '%s'", abs_path)
        extract_path = temp_dir / generate_random_string()
        extract_archive(
            archive_path=abs_path,
            extract_to=extract_path,
        )
        logger.debug("搜索解压出的文件夹: '%s'", extract_path)
        # 递归调用 (标记 is_toplevel=False)
        return find_desktop_entry_file(
            input_file=extract_path,
            temp_dir=temp_dir,
            depth=depth - 1,  # 避免压缩包嵌套造成死循环
            visited=visited,
            is_toplevel=False,
        )

    # 如果是文件夹则尝试遍历文件夹中的文件
    if abs_path.is_dir():
        logger.debug("搜索 '%s' 文件夹", abs_path)
        # 获取下级文件列表
        paths = get_file_list(
            path=abs_path,
            max_depth=0,
            include_dirs=True,
        )
        for path in paths:
            file = find_desktop_entry_file(
                input_file=path,
                temp_dir=temp_dir,
                depth=depth - 1,
                visited=visited,
                is_toplevel=False,
            )
            if file is not None:
                return file

    return None


def find_inf_file(
    input_file: Path,
    temp_dir: Path,
    depth: int | None = 0,
    visited: set[Path] | None = None,
    is_toplevel: bool = True,
) -> Path | None:
    """搜索 INF 文件路径

    Args:
        input_file (Path): 任意文件路径
        temp_dir (Path): 临时文件夹, 用于保存寻找文件时产生的临时文件
        depth (int | None): 递归搜索文件的深度
        visited (set[Path] | None): 已访问路径集合, 用于防止死循环
        is_toplevel (bool): 是否为顶层调用, 用于控制是否修正光标文件路径
    Returns:
        (Path | None): 当找到 INF 文件时则返回其路径, 否则返回 None
    """

    # 初始化去重集合
    if visited is None:
        visited = set()

    logger.debug("已搜索路径: '%s'", visited)
    logger.debug("查找路径: '%s'", input_file)

    # 统一去重检查 (针对 URL 或 Path)
    if input_file in visited:
        return None

    # 递归搜索深度控制
    if depth < 0:
        return None

    # 检测为下载链接时
    if is_http_or_https(input_file):
        visited.add(input_file)
        try:
            download_file = download_file_from_url(
                url=input_file,
                save_path=temp_dir,
            )
            if not is_supported_archive_format(download_file):
                logger.debug("下载的文件不是支持的压缩包格式: '%s'", download_file)
                return None

            logger.debug("从 '%s' 下载成功, 准备解压搜索: '%s'", input_file, download_file)
            extract_path = temp_dir / generate_random_string()
            extract_archive(
                archive_path=download_file,
                extract_to=extract_path,
            )
            # 递归调用 (标记 is_toplevel=False)
            return find_inf_file(
                input_file=extract_path,
                temp_dir=temp_dir,
                depth=depth - 1,  # 避免压缩包嵌套造成死循环
                visited=visited,
                is_toplevel=False,
            )
        except Exception as e:
            logger.debug("从 '%s' 下载文件时发生错误, 终止寻找: %s", input_file, e)
            return None

    # 智能路径修正 (仅在首次调用且输入为本地光标文件时触发)
    # 该目录可能为鼠标指针路径, 则尝试定位到其父文件夹
    if is_toplevel and isinstance(input_file, Path) and input_file.is_file():
        if input_file.name.lower().endswith((".ani", ".cur")):
            input_file = input_file.parent

    # 文件系统逻辑处理
    try:
        # 确保 input_file 是 Path 对象
        abs_path = Path(input_file).resolve()
    except Exception:
        return None

    if not abs_path.exists() or abs_path in visited:
        return None
    visited.add(abs_path)

    # 验证 INF 文件完整性
    if abs_path.is_file() and abs_path.name.lower().endswith(".inf"):
        try:
            _ = parse_inf_file_content(abs_path)
            logger.debug("搜索到 INF 文件路径: '%s'", abs_path)
            return abs_path
        except ValueError:
            return None

    # 文件为压缩包时则尝试解压并遍历解压的文件夹
    if is_supported_archive_format(abs_path):
        logger.debug("检测到压缩包，准备解压: '%s'", abs_path)
        extract_path = temp_dir / generate_random_string()
        extract_archive(
            archive_path=abs_path,
            extract_to=extract_path,
        )
        logger.debug("搜索解压出的文件夹: '%s'", extract_path)
        # 递归调用 (标记 is_toplevel=False)
        return find_inf_file(
            input_file=extract_path,
            temp_dir=temp_dir,
            depth=depth - 1,  # 避免压缩包嵌套造成死循环
            visited=visited,
            is_toplevel=False,
        )

    # 如果是文件夹则尝试遍历文件夹中的文件
    if abs_path.is_dir():
        logger.debug("搜索 '%s' 文件夹", abs_path)
        # 获取下级文件列表
        paths = get_file_list(
            path=abs_path,
            max_depth=0,
            include_dirs=True,
        )
        for path in paths:
            file = find_inf_file(
                input_file=path,
                temp_dir=temp_dir,
                depth=depth - 1,
                visited=visited,
                is_toplevel=False,
            )
            if file is not None:
                return file

    return None



"""XDG 环境配置工具"""

import configparser
from pathlib import Path

XDG_CONFIG_PATH = Path("~/.icons/default/index.theme").expanduser()
"""XDG 配置文件路径"""

XDG_CONFIG_SHARE_PATH = Path("~/.local/share/icons/default/index.theme").expanduser()
"""XDG 配置文件路径 (共享路径)"""


def get_xdg_cursor_theme() -> tuple[str, str] | None:
    """获取 XDG 标准方案的当前使用的鼠标指针配置名称

    Returns:
        (tuple[str, str] | None): 当前使用的鼠标指针名称
    """
    config = configparser.ConfigParser()
    config_share = configparser.ConfigParser()
    config.read(XDG_CONFIG_PATH)
    config_share.read(XDG_CONFIG_SHARE_PATH)
    cursor_name = config.get("Icon Theme", "Inherits") if "Icon Theme" in config and "Inherits" in config["Icon Theme"] else None
    cursor_name_share = config_share.get("Icon Theme", "Inherits") if "Icon Theme" in config_share and "Inherits" in config_share["Icon Theme"] else None
    return (cursor_name, cursor_name_share)


def set_xdg_cursor_theme(cursor_name: str) -> None:
    """设置 XDG 标准方案的当前使用的鼠标指针配置名称

    Args:
        cursor_name (str): 要设置的鼠标指针配置名称
    """
    XDG_CONFIG_PATH.parent.mkdir(parents=True, exist_ok=True)
    XDG_CONFIG_SHARE_PATH.parent.mkdir(parents=True, exist_ok=True)
    config = configparser.ConfigParser()
    config_share = configparser.ConfigParser()
    config.read(XDG_CONFIG_PATH)
    config_share.read(XDG_CONFIG_SHARE_PATH)
    if "Icon Theme" not in config:
        config["Icon Theme"] = {}

    if "Icon Theme" not in config_share:
        config_share["Icon Theme"] = {}

    config["Icon Theme"]["Inherits"] = cursor_name
    config_share["Icon Theme"]["Inherits"] = cursor_name

    with open(XDG_CONFIG_PATH, "w", encoding="utf-8") as f:
        config.write(f, space_around_delimiters=False)

    with open(XDG_CONFIG_SHARE_PATH, "w", encoding="utf-8") as f:
        config_share.write(f, space_around_delimiters=False)
